@@ -16,10 +16,19 @@ XTRACE=$(set +o | grep xtrace)
 set -o xtrace
 
 function container_runtime {
+    # Ignore error at killing/removing a container doesn't running to avoid
+    # unstack is terminated.
+    # TODO: Support for CRI-O if it's required.
+    local regex_cmds_ignore="(kill|rm)\s+"
+
     if [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
         sudo podman "$@" || die $LINENO "Error when running podman command"
     else
-        docker "$@" || die $LINENO "Error when running docker command"
+        if [[ $@ =~ $regex_cmds_ignore ]]; then
+            docker "$@"
+        else
+            docker "$@" || die $LINENO "Error when running docker command"
+        fi
     fi
 }
 
@@ -654,7 +663,8 @@ function run_k8s_controller_manager {
                 --root-ca-file=/srv/kubernetes/ca.crt
                 --min-resync-period=3m
                 --v=$(get_k8s_log_level)
-                --logtostderr=true)
+                --logtostderr=true
+                --leader-elect=false)
 
     run_container kubernetes-controller-manager "${command[@]}"
 }
@@ -674,7 +684,8 @@ function run_k8s_scheduler {
               /hyperkube kube-scheduler
                 --master=$KURYR_K8S_API_URL
                 --v=$(get_k8s_log_level)
-                --logtostderr=true)
+                --logtostderr=true
+                --leader-elect=false)
 
     run_container kubernetes-scheduler "${command[@]}"
 }
@@ -758,7 +769,7 @@ function run_k8s_kubelet {
         --root-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet"
 
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=" --cgroup-driver $(docker info|awk '/Cgroup/ {print $NF}')"
+        command+=" --cgroup-driver $(docker info -f '{{.CgroupDriver}}')"
     elif [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
         local crio_conf
         crio_conf=/etc/crio/crio.conf
@@ -854,7 +865,7 @@ spec:
       hostNetwork: true
       containers:
       - name: coredns
-        image: coredns/coredns:1.4.0
+        image: quay.io/kuryr/coredns:1.4.0
         imagePullPolicy: Always
         args: [ "-conf", "/etc/coredns/Corefile" ]
         volumeMounts:
