@@ -10,8 +10,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from http import client as httplib
 import os
-from six.moves import http_client as httplib
 
 from flask import Flask
 from pyroute2 import IPDB
@@ -127,11 +127,11 @@ class CNIHealthServer(object):
         data = 'ok'
         no_limit = -1
         try:
-            with IPDB() as a:
-                a.release()
+            with IPDB():
+                pass
         except Exception:
             error_message = 'IPDB not in working order.'
-            LOG.debug(error_message)
+            LOG.error(error_message)
             return error_message, httplib.INTERNAL_SERVER_ERROR, self.headers
 
         if CONF.cni_health_server.max_memory_usage != no_limit:
@@ -139,20 +139,20 @@ class CNIHealthServer(object):
 
             if mem_usage > CONF.cni_health_server.max_memory_usage:
                 err_message = 'CNI daemon exceeded maximum memory usage.'
-                LOG.debug(err_message)
+                LOG.error(err_message)
                 return err_message, httplib.INTERNAL_SERVER_ERROR, self.headers
 
         with self._components_healthy.get_lock():
             if not self._components_healthy.value:
                 err_message = 'Kuryr CNI components not healthy.'
-                LOG.debug(err_message)
+                LOG.error(err_message)
                 return err_message, httplib.INTERNAL_SERVER_ERROR, self.headers
 
         LOG.debug('Kuryr CNI Liveness verified.')
         return data, httplib.OK, self.headers
 
     def run(self):
-        address = '0.0.0.0'
+        address = '::'
         try:
             LOG.info('Starting CNI health check server.')
             self.application.run(address, CONF.cni_health_server.port)
