@@ -14,7 +14,8 @@
 
 import threading
 
-from neutronclient.common import exceptions as n_exc
+from openstack import exceptions as o_exc
+from oslo_config import cfg
 from oslo_log import log as logging
 
 from kuryr_kubernetes import clients
@@ -25,6 +26,7 @@ from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes import os_vif_util as ovu
 
 LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 
 class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
@@ -34,24 +36,24 @@ class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
         self.lock = threading.Lock()
 
     def request_vif(self, pod, project_id, subnets, security_groups):
-        neutron = clients.get_neutron_client()
+        os_net = clients.get_network_client()
         req = self._get_port_request(pod, project_id, subnets,
                                      security_groups)
         attempts = kuryr_config.CONF.pod_vif_nested.rev_update_attempts
         container_port = None
         while attempts > 0:
-            vm_port = self._get_parent_port(neutron, pod)
+            vm_port = self._get_parent_port(pod)
 
             if not container_port:
-                container_port = neutron.create_port(req).get('port')
-            utils.tag_neutron_resources('ports', [container_port['id']])
+                container_port = os_net.create_port(**req)
+            utils.tag_neutron_resources([container_port])
 
-            container_mac = container_port['mac_address']
+            container_mac = container_port.mac_address
             container_ips = frozenset(entry['ip_address'] for entry in
-                                      container_port['fixed_ips'])
+                                      container_port.fixed_ips)
 
             attempts = self._try_update_port(
-                attempts, self._add_to_allowed_address_pairs, neutron, vm_port,
+                attempts, self._add_to_allowed_address_pairs, vm_port,
                 container_ips, container_mac)
 
         return ovu.neutron_to_osvif_vif_nested_macvlan(container_port, subnets)
@@ -62,23 +64,23 @@ class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
         raise NotImplementedError()
 
     def release_vif(self, pod, vif, project_id=None, security_groups=None):
-        neutron = clients.get_neutron_client()
+        os_net = clients.get_network_client()
 
         attempts = kuryr_config.CONF.pod_vif_nested.rev_update_attempts
         while attempts > 0:
-            container_port = neutron.show_port(vif.id).get('port')
+            container_port = os_net.get_port(vif.id)
 
-            container_mac = container_port['mac_address']
+            container_mac = container_port.mac_address
             container_ips = frozenset(entry['ip_address'] for entry in
-                                      container_port['fixed_ips'])
-            vm_port = self._get_parent_port(neutron, pod)
+                                      container_port.fixed_ips)
+            vm_port = self._get_parent_port(pod)
             attempts = self._try_update_port(
-                attempts, self._remove_from_allowed_address_pairs, neutron,
+                attempts, self._remove_from_allowed_address_pairs,
                 vm_port, container_ips, container_mac)
 
         try:
-            neutron.delete_port(vif.id)
-        except n_exc.PortNotFoundClient:
+            os_net.delete_port(vif.id, ignore_missing=False)
+        except o_exc.ResourceNotFound:
             LOG.warning("Unable to release port %s as it no longer exists.",
                         vif.id)
 
@@ -90,16 +92,16 @@ class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
         # immediately to let the CNI driver make progress.
         vif.active = True
 
-    def _add_to_allowed_address_pairs(self, neutron, port, ip_addresses,
+    def _add_to_allowed_address_pairs(self, port, ip_addresses,
                                       mac_address=None):
         if not ip_addresses:
             raise k_exc.IntegrityError(
                 "Cannot add pair from the "
                 "allowed_address_pairs of port %s: missing IP address",
-                port['id'])
+                port.id)
 
-        mac = mac_address if mac_address else port['mac_address']
-        address_pairs = port['allowed_address_pairs']
+        mac = mac_address if mac_address else port.mac_address
+        address_pairs = port.allowed_address_pairs
 
         # look for duplicates or near-matches
         for pair in address_pairs:
@@ -120,22 +122,22 @@ class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
             address_pairs.append({'ip_address': ip, 'mac_address': mac})
 
         self._update_port_address_pairs(
-            neutron, port['id'], address_pairs,
-            revision_number=port['revision_number'])
+            port.id, address_pairs,
+            revision_number=port.revision_number)
 
         LOG.debug("Added allowed_address_pair %s %s" %
                   (str(ip_addresses,), mac_address))
 
-    def _remove_from_allowed_address_pairs(self, neutron, port, ip_addresses,
+    def _remove_from_allowed_address_pairs(self, port, ip_addresses,
                                            mac_address=None):
         if not ip_addresses:
             raise k_exc.IntegrityError(
                 "Cannot remove pair from the "
                 "allowed_address_pairs of port %s: missing IP address",
-                port['id'])
+                port.id)
 
-        mac = mac_address if mac_address else port['mac_address']
-        address_pairs = port['allowed_address_pairs']
+        mac = mac_address if mac_address else port.mac_address
+        address_pairs = port.allowed_address_pairs
         updated = False
 
         for ip in ip_addresses:
@@ -149,25 +151,23 @@ class NestedMacvlanPodVIFDriver(nested_vif.NestedPodVIFDriver):
 
         if updated:
             self._update_port_address_pairs(
-                neutron, port['id'],
+                port.id,
                 address_pairs,
-                revision_number=port['revision_number'])
+                revision_number=port.revision_number)
 
-    def _update_port_address_pairs(self, neutron, port_id, address_pairs,
+    def _update_port_address_pairs(self, port_id, address_pairs,
                                    revision_number=None):
-        neutron.update_port(
-            port_id,
-            {'port': {'allowed_address_pairs': address_pairs}},
-            revision_number=revision_number
-        )
+        os_net = clients.get_network_client()
+        os_net.update_port(port_id, allowed_address_pairs=address_pairs,
+                           if_match=f'revision_number={revision_number}')
 
     def _try_update_port(self, attempts, f,
-                         neutron, vm_port, container_ips, container_mac):
+                         vm_port, container_ips, container_mac):
         try:
             with self.lock:
-                f(neutron, vm_port, container_ips, container_mac)
+                f(vm_port, container_ips, container_mac)
                 attempts = 0
-        except n_exc.NeutronClientException:
+        except o_exc.SDKException:
             attempts -= 1
             if attempts == 0:
                 LOG.exception("Error happened during updating port %s",
