@@ -180,6 +180,10 @@ def _create_sg_rule_on_text_port(sg_id, direction, port, rule_selected_pods,
                 pods=pods)
         else:
             namespace_obj = driver_utils.get_namespace(namespace)
+            if not namespace_obj:
+                LOG.debug("Skipping SG rule creation. Inexistent"
+                          " namespace.")
+                continue
             namespace_cidr = driver_utils.get_namespace_subnet_cidr(
                 namespace_obj)
             sg_rule = driver_utils.create_security_group_rule_body(
@@ -188,7 +192,8 @@ def _create_sg_rule_on_text_port(sg_id, direction, port, rule_selected_pods,
                 pods=pods)
         sgr_id = driver_utils.create_security_group_rule(sg_rule)
         sg_rule['security_group_rule']['id'] = sgr_id
-        crd_rules.append(sg_rule)
+        if sg_rule not in crd_rules:
+            crd_rules.append(sg_rule)
     return matched
 
 
@@ -218,12 +223,14 @@ def _create_sg_rules(crd, pod, pod_selector, rule_block,
                         sg_rule = _create_sg_rule(
                             sg_id, direction, cidr=pod_ip, port=port,
                             namespace=namespace)
-                        crd_rules.append(sg_rule)
+                        if sg_rule not in crd_rules:
+                            crd_rules.append(sg_rule)
             else:
                 matched = True
                 sg_rule = _create_sg_rule(
                     sg_id, direction, cidr=pod_ip, namespace=namespace)
-                crd_rules.append(sg_rule)
+                if sg_rule not in crd_rules:
+                    crd_rules.append(sg_rule)
     else:
         # NOTE (maysams) When a policy with namespaceSelector and text port
         # is applied the port on the pods needs to be retrieved.
@@ -292,9 +299,11 @@ def _parse_selectors_on_namespace(crd, direction, pod_selector,
                                 LOG.debug("Skipping SG rule creation for pod "
                                           "%s due to no IP assigned", pod_name)
                                 continue
-                            crd_rules.append(_create_sg_rule(
+                            sg_rule = _create_sg_rule(
                                 sg_id, direction, pod_ip, port=port,
-                                namespace=ns_name))
+                                namespace=ns_name)
+                            if sg_rule not in crd_rules:
+                                crd_rules.append(sg_rule)
             else:
                 for pod in pods:
                     pod_ip = driver_utils.get_pod_ip(pod)
@@ -304,9 +313,11 @@ def _parse_selectors_on_namespace(crd, direction, pod_selector,
                                   " to no IP assigned", pod_name)
                         continue
                     matched = True
-                    crd_rules.append(_create_sg_rule(
+                    sg_rule = _create_sg_rule(
                         sg_id, direction, pod_ip,
-                        namespace=ns_name))
+                        namespace=ns_name)
+                    if sg_rule not in crd_rules:
+                        crd_rules.append(sg_rule)
         else:
             ns_pods = driver_utils.get_pods(ns_selector)['items']
             ns_cidr = driver_utils.get_namespace_subnet_cidr(namespace)
@@ -319,14 +330,18 @@ def _parse_selectors_on_namespace(crd, direction, pod_selector,
                                 crd_rules, matched, crd))
                     else:
                         matched = True
-                        crd_rules.append(_create_sg_rule(
+                        sg_rule = _create_sg_rule(
                             sg_id, direction, ns_cidr,
-                            port=port, namespace=ns_name))
+                            port=port, namespace=ns_name)
+                        if sg_rule not in crd_rules:
+                            crd_rules.append(sg_rule)
             else:
                 matched = True
-                crd_rules.append(_create_sg_rule(
+                sg_rule = _create_sg_rule(
                     sg_id, direction, ns_cidr,
-                    namespace=ns_name))
+                    namespace=ns_name)
+                if sg_rule not in crd_rules:
+                    crd_rules.append(sg_rule)
     return matched, crd_rules
 
 
@@ -381,13 +396,14 @@ def _parse_rules_on_delete_namespace(rule_list, direction, ns_name):
             matched = True
             driver_utils.delete_security_group_rule(
                 rule['security_group_rule']['id'])
-        for remote_ip, namespace in list(remote_ip_prefixes.items()):
-            if namespace == ns_name:
-                matched = True
-                remote_ip_prefixes.pop(remote_ip)
-                if remote_ip_prefixes:
-                    rule['remote_ip_prefixes'] = remote_ip_prefixes
-                    rules.append(rule)
+        elif remote_ip_prefixes:
+            for remote_ip, namespace in list(remote_ip_prefixes.items()):
+                if namespace == ns_name:
+                    matched = True
+                    remote_ip_prefixes.pop(remote_ip)
+            if remote_ip_prefixes:
+                rule['remote_ip_prefixes'] = remote_ip_prefixes
+                rules.append(rule)
         else:
             rules.append(rule)
     return matched, rules
@@ -559,15 +575,6 @@ class NetworkPolicySecurityGroupsDriver(base.PodSecurityGroupsDriver):
         crd_selectors.extend(self.create_namespace_sg_rules(namespace))
         return crd_selectors
 
-    def create_namespace_sg(self, namespace, project_id, crd_spec):
-        LOG.debug("Security group driver does not create SGs for the "
-                  "namespaces.")
-        return {}
-
-    def delete_sg(self, sg_id):
-        LOG.debug("Security group driver does not implement deleting "
-                  "SGs.")
-
 
 class NetworkPolicyServiceSecurityGroupsDriver(
         base.ServiceSecurityGroupsDriver):
