@@ -13,19 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_cache import core as cache
+import urllib
+
+from openstack import exceptions as os_exc
 from oslo_config import cfg
 from oslo_log import log
 from oslo_serialization import jsonutils
-from six.moves.urllib import parse
 
 from kuryr_kubernetes import clients
 from kuryr_kubernetes import constants
 from kuryr_kubernetes import exceptions as k_exc
-from kuryr_kubernetes import os_vif_util as ovu
 from kuryr_kubernetes import utils
 
-from neutronclient.common import exceptions as n_exc
 
 OPERATORS_WITH_VALUES = [constants.K8S_OPERATOR_IN,
                          constants.K8S_OPERATOR_NOT_IN]
@@ -34,23 +33,9 @@ LOG = log.getLogger(__name__)
 
 CONF = cfg.CONF
 
-pod_ip_caching_opts = [
-    cfg.BoolOpt('caching', default=True),
-    cfg.IntOpt('cache_time', default=3600),
-]
-
-CONF.register_opts(pod_ip_caching_opts, "pod_ip_caching")
-
-cache.configure(CONF)
-pod_ip_cache_region = cache.create_region()
-MEMOIZE = cache.get_memoization_decorator(
-    CONF, pod_ip_cache_region, "pod_ip_caching")
-
-cache.configure_cache_region(CONF, pod_ip_cache_region)
-
 
 def get_network_id(subnets):
-    ids = ovu.osvif_to_neutron_network_ids(subnets)
+    ids = list(set(net.id for net in subnets.values()))
 
     if len(ids) != 1:
         raise k_exc.IntegrityError(
@@ -118,10 +103,10 @@ def get_pods(selector, namespace=None):
         if exps:
             exps = ', '.join(format_expression(exp) for exp in exps)
             if labels:
-                expressions = parse.quote("," + exps)
+                expressions = urllib.parse.quote("," + exps)
                 labels += expressions
             else:
-                labels = parse.quote(exps)
+                labels = urllib.parse.quote(exps)
 
     if namespace:
         pods = kubernetes.get(
@@ -150,10 +135,10 @@ def get_namespaces(selector):
     if exps:
         exps = ', '.join(format_expression(exp) for exp in exps)
         if labels:
-            expressions = parse.quote("," + exps)
+            expressions = urllib.parse.quote("," + exps)
             labels += expressions
         else:
-            labels = parse.quote(exps)
+            labels = urllib.parse.quote(exps)
 
     namespaces = kubernetes.get(
         '{}/namespaces?labelSelector={}'.format(
@@ -178,7 +163,7 @@ def format_expression(expression):
 
 
 def replace_encoded_characters(labels):
-    labels = parse.urlencode(labels)
+    labels = urllib.parse.urlencode(labels)
     # NOTE(ltomasbo): K8s API does not accept &, so we need to AND
     # the matchLabels with ',' or '%2C' instead
     labels = labels.replace('&', ',')
@@ -186,50 +171,41 @@ def replace_encoded_characters(labels):
 
 
 def create_security_group_rule(body):
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
+
     sgr = ''
+
     try:
-        sgr = neutron.create_security_group_rule(
-            body=body)
-    except n_exc.Conflict as ex:
+        params = dict(body['security_group_rule'])
+        if 'ethertype' in params:
+            # NOTE(gryf): in openstacksdk, there is ether_type attribute in
+            # the security_group_rule object, in CRD we have 'ethertype'
+            # instead, just like it was returned by the neutron client.
+            params['ether_type'] = params['ethertype']
+            del params['ethertype']
+        sgr = os_net.create_security_group_rule(**params)
+        return sgr.id
+    except os_exc.ConflictException as ex:
         LOG.debug("Failed to create already existing security group "
                   "rule %s", body)
         # Get existent sg rule id from exception message
-        sgr_id = str(ex).split("Rule id is", 1)[1].split()[0][:-1]
-        return sgr_id
-    except n_exc.NeutronClientException:
+        return str(ex).split()[-1][:-1]
+    except os_exc.SDKException:
         LOG.debug("Error creating security group rule")
         raise
-    return sgr["security_group_rule"]["id"]
 
 
 def delete_security_group_rule(security_group_rule_id):
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
     try:
         LOG.debug("Deleting sg rule with ID: %s", security_group_rule_id)
-        neutron.delete_security_group_rule(
-            security_group_rule=security_group_rule_id)
-    except n_exc.NotFound:
-        LOG.debug("Error deleting security group rule as it does not "
-                  "exist: %s", security_group_rule_id)
-    except n_exc.NeutronClientException:
+        os_net.delete_security_group_rule(security_group_rule_id)
+    except os_exc.SDKException:
         LOG.debug("Error deleting security group rule: %s",
                   security_group_rule_id)
         raise
 
 
-def patch_kuryrnet_crd(crd, populated=True):
-    kubernetes = clients.get_kubernetes_client()
-    crd_name = crd['metadata']['name']
-    LOG.debug('Patching KuryrNet CRD %s' % crd_name)
-    try:
-        kubernetes.patch_crd('spec', crd['metadata']['selfLink'],
-                             {'populated': populated})
-    except k_exc.K8sClientException:
-        LOG.exception('Error updating kuryrnet CRD %s', crd_name)
-        raise
-
-
 def patch_kuryrnetworkpolicy_crd(crd, i_rules, e_rules, pod_selector,
                                  np_spec=None):
     kubernetes = clients.get_kubernetes_client()
@@ -263,19 +239,19 @@ def create_security_group_rule_body(
     if not protocol:
         protocol = 'TCP'
     security_group_rule_body = {
-        u'security_group_rule': {
-            u'ethertype': ethertype,
-            u'security_group_id': security_group_id,
-            u'description': description,
-            u'direction': direction,
-            u'protocol': protocol.lower(),
-            u'port_range_min': port_range_min,
-            u'port_range_max': port_range_max,
+        'security_group_rule': {
+            'ethertype': ethertype,
+            'security_group_id': security_group_id,
+            'description': description,
+            'direction': direction,
+            'protocol': protocol.lower(),
+            'port_range_min': port_range_min,
+            'port_range_max': port_range_max,
         }
     }
     if cidr:
-        security_group_rule_body[u'security_group_rule'][
-            u'remote_ip_prefix'] = cidr
+        security_group_rule_body['security_group_rule'][
+            'remote_ip_prefix'] = cidr
     if namespace:
         security_group_rule_body['namespace'] = namespace
     if pods:
@@ -284,7 +260,6 @@ def create_security_group_rule_body(
     return security_group_rule_body
 
 
-@MEMOIZE
 def get_pod_ip(pod):
     try:
         pod_metadata = pod['metadata']['annotations']
@@ -347,7 +322,7 @@ def match_expressions(expressions, labels):
                 label_value = labels.get(str(exp['key']), None)
                 if exp_op == constants.K8S_OPERATOR_IN:
                     if label_value is None or label_value not in exp_values:
-                            return False
+                        return False
                 elif exp_op == constants.K8S_OPERATOR_NOT_IN:
                     if label_value in exp_values:
                         return False
@@ -371,19 +346,22 @@ def match_labels(crd_labels, labels):
     for crd_key, crd_value in crd_labels.items():
         label_value = labels.get(crd_key, None)
         if not label_value or crd_value != label_value:
-                return False
+            return False
     return True
 
 
 def match_selector(selector, labels):
+    if selector is None:
+        return True
+    if labels is None:
+        labels = {}
     crd_labels = selector.get('matchLabels', None)
     crd_expressions = selector.get('matchExpressions', None)
 
     match_exp = match_lb = True
     if crd_expressions:
-        match_exp = match_expressions(crd_expressions,
-                                      labels)
-    if crd_labels and labels:
+        match_exp = match_expressions(crd_expressions, labels)
+    if crd_labels:
         match_lb = match_labels(crd_labels, labels)
     return match_exp and match_lb
 
@@ -391,32 +369,40 @@ def match_selector(selector, labels):
 def get_namespace_subnet_cidr(namespace):
     kubernetes = clients.get_kubernetes_client()
     try:
-        ns_annotations = namespace['metadata']['annotations']
-        ns_name = ns_annotations[constants.K8S_ANNOTATION_NET_CRD]
-    except KeyError:
-        LOG.exception('Namespace handler must be enabled to support '
-                      'Network Policies with namespaceSelector')
+        net_crd_path = (f"{constants.K8S_API_CRD_NAMESPACES}/"
+                        f"{namespace['metadata']['name']}/kuryrnetworks/"
+                        f"{namespace['metadata']['name']}")
+        net_crd = kubernetes.get(net_crd_path)
+    except k_exc.K8sResourceNotFound:
+        LOG.exception('Namespace not yet ready')
         raise k_exc.ResourceNotReady(namespace)
-    try:
-        net_crd = kubernetes.get('{}/kuryrnets/{}'.format(
-            constants.K8S_API_CRD, ns_name))
     except k_exc.K8sClientException:
         LOG.exception("Kubernetes Client Exception.")
         raise
-    return net_crd['spec']['subnetCIDR']
+    try:
+        subnet_cidr = net_crd['status']['subnetCIDR']
+    except KeyError:
+        LOG.exception('Namespace not yet ready')
+        raise k_exc.ResourceNotReady(namespace)
+    return subnet_cidr
 
 
-def tag_neutron_resources(resource, res_ids):
+def tag_neutron_resources(resources):
+    """Set tags to the provided resources.
+
+    param resources: list of openstacksdk objects to tag.
+    """
     tags = CONF.neutron_defaults.resource_tags
-    if tags:
-        neutron = clients.get_neutron_client()
-        for res_id in res_ids:
-            try:
-                neutron.replace_tag(resource, res_id, body={"tags": tags})
-            except n_exc.NeutronClientException:
-                LOG.warning("Failed to tag %s %s with %s. Ignoring, but this "
-                            "is still unexpected.", resource, res_id, tags,
-                            exc_info=True)
+    if not tags:
+        return
+
+    os_net = clients.get_network_client()
+    for res in resources:
+        try:
+            os_net.set_tags(res, tags=tags)
+        except os_exc.SDKException:
+            LOG.warning("Failed to tag %s with %s. Ignoring, but this is "
+                        "still unexpected.", res, tags, exc_info=True)
 
 
 def get_services(namespace=None):
@@ -509,24 +495,23 @@ def get_ports(resource, port):
 
 def get_namespace(namespace_name):
     kubernetes = clients.get_kubernetes_client()
-    return kubernetes.get(
-        '{}/namespaces/{}'.format(
-            constants.K8S_API_BASE, namespace_name))
+    try:
+        return kubernetes.get(
+            '{}/namespaces/{}'.format(
+                constants.K8S_API_BASE, namespace_name))
+    except k_exc.K8sResourceNotFound:
+        LOG.debug("Namespace not found: %s",
+                  namespace_name)
+        return None
 
 
 def update_port_pci_info(pod, vif):
     node = get_host_id(pod)
     annot_port_pci_info = get_port_annot_pci_info(node, vif.id)
-    neutron = clients.get_neutron_client()
+    os_net = clients.get_network_client()
     LOG.debug("Neutron port %s is updated with binding:profile info %s",
               vif.id, annot_port_pci_info)
-    neutron.update_port(
-        vif.id,
-        {
-            "port": {
-                'binding:profile': annot_port_pci_info
-            }
-        })
+    os_net.update_port(vif.id, binding_profile=annot_port_pci_info)
 
 
 def get_port_annot_pci_info(nodename, neutron_port):
@@ -545,9 +530,3 @@ def get_port_annot_pci_info(nodename, neutron_port):
         LOG.exception('Exception when reading annotations '
                       '%s and converting from json', annot_name)
     return pci_info
-
-
-def get_ports_by_attrs(**attrs):
-    neutron = clients.get_neutron_client()
-    ports = neutron.list_ports(**attrs)
-    return ports['ports']
