@@ -13,6 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import eventlet
+import time
+
 from kuryr.lib._i18n import _
 from oslo_log import log as logging
 
@@ -20,6 +23,7 @@ from kuryr_kubernetes import clients
 from kuryr_kubernetes import config
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drv_base
+from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.handlers import k8s_base
 from kuryr_kubernetes.objects import lbaas as obj_lbaas
@@ -159,6 +163,8 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         self._drv_pod_project = drv_base.PodProjectDriver.get_instance()
         self._drv_pod_subnets = drv_base.PodSubnetsDriver.get_instance()
         self._drv_service_pub_ip = drv_base.ServicePubIpDriver.get_instance()
+        self._drv_project = drv_base.ServiceProjectDriver.get_instance()
+        self._drv_sg = drv_base.ServiceSecurityGroupsDriver.get_instance()
         # Note(yboaron) LBaaS driver supports 'provider' parameter in
         # Load Balancer creation flow.
         # We need to set the requested load balancer provider
@@ -168,6 +174,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 != 'default'):
             self._lb_provider = (
                 config.CONF.kubernetes.endpoints_driver_octavia_provider)
+        eventlet.spawn(self._cleanup_leftover_lbaas)
 
     def on_present(self, endpoints):
         lbaas_spec = utils.get_lbaas_spec(endpoints)
@@ -276,37 +283,28 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         return changed
 
-    def _sync_lbaas_sgs(self, endpoints, lbaas_state, lbaas_spec):
-        # NOTE (maysams) Need to retrieve the LBaaS Spec again due to
-        # the possibility of it being updated after the LBaaS creation
-        # process has started.
+    def _sync_lbaas_sgs(self, endpoints, lbaas_state):
         svc_link = self._get_service_link(endpoints)
         k8s = clients.get_kubernetes_client()
         service = k8s.get(svc_link)
-        lbaas_spec = utils.get_lbaas_spec(service)
 
         lb = lbaas_state.loadbalancer
-        default_sgs = config.CONF.neutron_defaults.pod_security_groups
-        # NOTE(maysams) As the endpoint and svc are annotated with the
-        # 'lbaas_spec' in two separate k8s calls, it's possible that
-        # the endpoint got annotated and the svc haven't due to controller
-        # restarts. For this case, a resourceNotReady exception is raised
-        # till the svc gets annotated with a 'lbaas_spec'.
-        if lbaas_spec:
-            lbaas_spec_sgs = lbaas_spec.security_groups_ids
-        else:
-            raise k_exc.ResourceNotReady(svc_link)
-        if lb.security_groups and lb.security_groups != lbaas_spec_sgs:
-            sgs = [lb_sg for lb_sg in lb.security_groups
-                   if lb_sg not in default_sgs]
-            if lbaas_spec_sgs != default_sgs:
-                sgs.extend(lbaas_spec_sgs)
-            lb.security_groups = sgs
+        # NOTE(maysams) It's possible that while the service annotation
+        # is added the backend pods on that service are not yet created
+        # resulting in no security groups retrieved for the service.
+        # Let's retrieve again to ensure is updated.
+        project_id = self._drv_project.get_project(service)
+        lb_sgs = self._drv_sg.get_security_groups(service, project_id)
+        lb.security_groups = lb_sgs
 
     def _add_new_members(self, endpoints, lbaas_state, lbaas_spec):
         changed = False
 
-        self._sync_lbaas_sgs(endpoints, lbaas_state, lbaas_spec)
+        try:
+            self._sync_lbaas_sgs(endpoints, lbaas_state)
+        except k_exc.K8sResourceNotFound:
+            LOG.debug("The svc has been deleted while processing the endpoints"
+                      " update. No need to add new members.")
 
         lsnr_by_id = {l.id: l for l in lbaas_state.listeners}
         pool_by_lsnr_port = {(lsnr_by_id[p.listener_id].protocol,
@@ -356,8 +354,14 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     # octavia does not require it
                     if (config.CONF.octavia_defaults.member_mode ==
                             k_const.OCTAVIA_L2_MEMBER_MODE):
-                        member_subnet_id = self._get_pod_subnet(target_ref,
-                                                                target_ip)
+                        try:
+                            member_subnet_id = self._get_pod_subnet(target_ref,
+                                                                    target_ip)
+                        except k_exc.K8sResourceNotFound:
+                            LOG.debug("Member namespace has been deleted. No "
+                                      "need to add the members as it is "
+                                      "going to be deleted")
+                            continue
                     else:
                         # We use the service subnet id so that the connectivity
                         # from VIP to pods happens in layer 3 mode, i.e.,
@@ -421,8 +425,8 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
             if port:
                 spec_ports[port.name] = pool.id
 
-        current_targets = {(a['ip'], p['port'],
-                            spec_ports.get(p.get('name')))
+        current_targets = {(a['ip'], a.get('targetRef', {}).get('name', ''),
+                            p['port'], spec_ports.get(p.get('name')))
                            for s in endpoints['subsets']
                            for a in s['addresses']
                            for p in s['ports']
@@ -430,7 +434,14 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         removed_ids = set()
         for member in lbaas_state.members:
-            if ((str(member.ip), member.port, member.pool_id) in
+            try:
+                member_name = member.name
+                # NOTE: The member name is compose of:
+                # NAMESPACE_NAME/POD_NAME:PROTOCOL_PORT
+                pod_name = member_name.split('/')[1].split(':')[0]
+            except AttributeError:
+                pod_name = ""
+            if ((str(member.ip), pod_name, member.port, member.pool_id) in
                     current_targets):
                 continue
             self._drv_lbaas.release_member(lbaas_state.loadbalancer,
@@ -519,17 +530,19 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         for port_spec in lbaas_spec_ports:
             protocol = port_spec.protocol
             port = port_spec.port
+            name = "%s:%s" % (lbaas_state.loadbalancer.name, protocol)
+            listener = [l for l in lbaas_state.listeners
+                        if l.port == port and l.protocol == protocol]
+            if listener:
+                continue
             # FIXME (maysams): Due to a bug in Octavia, which does
             # not allows listeners with same port but different
             # protocols to co-exist, we need to skip the creation of
             # listeners that have the same port as an existing one.
-            name = "%s:%s" % (lbaas_state.loadbalancer.name, protocol)
-            listener = self._get_listener_with_same_port(lbaas_state, port)
-            if listener:
-                if listener.protocol != protocol:
-                    LOG.warning("Skipping listener creation for %s "
-                                "as another one already exists with port %r",
-                                name, port)
+            listener = [l for l in lbaas_state.listeners if l.port == port]
+            if listener and not self._drv_lbaas.double_listeners_supported():
+                LOG.warning("Skipping listener creation for %s as another one"
+                            " already exists with port %s", name, port)
                 continue
             listener = self._drv_lbaas.ensure_listener(
                 loadbalancer=lbaas_state.loadbalancer,
@@ -541,12 +554,6 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                 changed = True
         return changed
 
-    def _get_listener_with_same_port(self, lbaas_state, port):
-        for listener in lbaas_state.listeners:
-            if listener.port == port:
-                return listener
-        return None
-
     def _remove_unused_listeners(self, endpoints, lbaas_state, lbaas_spec):
         current_listeners = {p.listener_id for p in lbaas_state.pools}
 
@@ -566,12 +573,12 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         status_data = {"loadBalancer": {
             "ingress": [{"ip": lb_ip_address.format()}]}}
         k8s = clients.get_kubernetes_client()
-        svc_link = self._get_service_link(endpoints)
+        svc_status_link = self._get_service_link(endpoints) + '/status'
         try:
-            k8s.patch("status", svc_link, status_data)
+            k8s.patch("status", svc_status_link, status_data)
         except k_exc.K8sClientException:
             # REVISIT(ivc): only raise ResourceNotReady for NotFound
-            raise k_exc.ResourceNotReady(svc_link)
+            raise k_exc.ResourceNotReady(svc_status_link)
 
     def _get_service_link(self, endpoints):
         ep_link = endpoints['metadata']['selfLink']
@@ -624,3 +631,45 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
 
         lbaas_state.loadbalancer = lb
         return changed
+
+    def _cleanup_leftover_lbaas(self):
+        lbaas_client = clients.get_loadbalancer_client()
+        services = []
+        try:
+            services = driver_utils.get_services().get('items')
+        except k_exc.K8sClientException:
+            LOG.debug("Skipping cleanup of leftover lbaas. "
+                      "Error retriving Kubernetes services")
+            return
+        services_cluster_ip = set(service['spec']['clusterIP']
+                                  for service in services
+                                  if service['spec'].get('clusterIP'))
+        lbaas_spec = {}
+        self._drv_lbaas.add_tags('loadbalancer', lbaas_spec)
+        loadbalancers = lbaas_client.load_balancers(**lbaas_spec)
+        for loadbalancer in loadbalancers:
+            if loadbalancer.vip_address not in services_cluster_ip:
+                lb_obj = obj_lbaas.LBaaSLoadBalancer(**loadbalancer)
+                eventlet.spawn(self._ensure_release_lbaas, lb_obj)
+
+    def _ensure_release_lbaas(self, lb_obj):
+        attempts = 0
+        deadline = 0
+        retry = True
+        timeout = config.CONF.kubernetes.watch_retry_timeout
+        while retry:
+            try:
+                if attempts == 1:
+                    deadline = time.time() + timeout
+                if (attempts > 0 and
+                        utils.exponential_sleep(deadline, attempts) == 0):
+                    LOG.error("Failed releasing lbaas '%s': deadline exceeded",
+                              lb_obj.name)
+                    return
+                self._drv_lbaas.release_loadbalancer(lb_obj)
+                retry = False
+            except k_exc.ResourceNotReady:
+                LOG.debug("Attempt (%s) of loadbalancer release %s failed."
+                          " A retry will be triggered.", attempts, lb_obj.name)
+                attempts += 1
+                retry = True
