@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from oslo_cache import core as cache
+from openstack import exceptions as os_exc
 from oslo_config import cfg as oslo_cfg
 from oslo_log import log as logging
 
@@ -20,26 +20,12 @@ from kuryr_kubernetes import clients
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drivers
 from kuryr_kubernetes.controller.drivers import utils as driver_utils
+from kuryr_kubernetes import exceptions
 from kuryr_kubernetes.handlers import k8s_base
 from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
 
-np_handler_caching_opts = [
-    oslo_cfg.BoolOpt('caching', default=True),
-    oslo_cfg.IntOpt('cache_time', default=120),
-]
-
-oslo_cfg.CONF.register_opts(np_handler_caching_opts,
-                            "np_handler_caching")
-
-cache.configure(oslo_cfg.CONF)
-np_handler_cache_region = cache.create_region()
-MEMOIZE = cache.get_memoization_decorator(
-    oslo_cfg.CONF, np_handler_cache_region, "np_handler_caching")
-
-cache.configure_cache_region(oslo_cfg.CONF, np_handler_cache_region)
-
 
 class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
     """NetworkPolicyHandler handles k8s Network Policies events"""
@@ -78,7 +64,8 @@ class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
             self._drv_vif_pool.update_vif_sgs(pod, pod_sgs)
 
         if (pods_to_update and
-                oslo_cfg.CONF.octavia_defaults.enforce_sg_rules):
+                oslo_cfg.CONF.octavia_defaults.enforce_sg_rules and
+                not self._is_egress_only_policy(policy)):
             # NOTE(ltomasbo): only need to change services if the pods that
             # they point to are updated
             services = driver_utils.get_services(
@@ -86,7 +73,7 @@ class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
             for service in services.get('items'):
                 # TODO(ltomasbo): Skip other services that are not affected
                 # by the policy
-                if (service['metadata']['name'] == 'kubernetes' or not
+                if (not service['spec'].get('selector') or not
                         self._is_service_affected(service, pods_to_update)):
                     continue
                 sgs = self._drv_svc_sg.get_security_groups(service,
@@ -114,15 +101,25 @@ class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
                         raise oslo_cfg.RequiredOptError(
                             'pod_security_groups',
                             oslo_cfg.OptGroup('neutron_defaults'))
-                self._drv_vif_pool.update_vif_sgs(pod, pod_sgs)
+                try:
+                    self._drv_vif_pool.update_vif_sgs(pod, pod_sgs)
+                except os_exc.NotFoundException:
+                    LOG.debug("Fail to update pod sgs."
+                              " Retrying policy deletion.")
+                    raise exceptions.ResourceNotReady(policy)
+
+            # ensure ports at the pool don't have the NP sg associated
+            net_id = self._get_policy_net_id(policy)
+            self._drv_vif_pool.remove_sg_from_pools(crd_sg, net_id)
 
             self._drv_policy.release_network_policy(netpolicy_crd)
 
-            if oslo_cfg.CONF.octavia_defaults.enforce_sg_rules:
+            if (oslo_cfg.CONF.octavia_defaults.enforce_sg_rules and
+                    not self._is_egress_only_policy(policy)):
                 services = driver_utils.get_services(
                     policy['metadata']['namespace'])
                 for svc in services.get('items'):
-                    if (svc['metadata']['name'] == 'kubernetes' or not
+                    if (not svc['spec'].get('selector') or not
                             self._is_service_affected(svc, pods_to_update)):
                         continue
                     sgs = self._drv_svc_sg.get_security_groups(svc,
@@ -134,13 +131,9 @@ class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
             return False
         return self._check_quota(quota)
 
-    @MEMOIZE
     def _check_quota(self, quota):
-        neutron = clients.get_neutron_client()
-        sg_quota = quota['security_group']
-        sg_func = neutron.list_security_groups
-        if utils.has_limit(sg_quota):
-            return utils.is_available('security_groups', sg_quota, sg_func)
+        if utils.has_limit(quota.security_groups):
+            return utils.is_available('security_groups', quota.security_groups)
         return True
 
     def _is_service_affected(self, service, affected_pods):
@@ -149,3 +142,22 @@ class NetworkPolicyHandler(k8s_base.ResourceEventHandler):
         svc_pods = driver_utils.get_pods({'selector': svc_selector},
                                          svc_namespace).get('items')
         return any(pod in svc_pods for pod in affected_pods)
+
+    def _get_policy_net_id(self, policy):
+        policy_ns = policy['metadata']['namespace']
+
+        kubernetes = clients.get_kubernetes_client()
+        try:
+            path = (f'{k_const.K8S_API_CRD_NAMESPACES}/{policy_ns}/'
+                    f'kuryrnetworks/{policy_ns}')
+            net_crd = kubernetes.get(path)
+        except exceptions.K8sClientException:
+            LOG.exception("Kubernetes Client Exception.")
+            raise
+        return net_crd['status']['netId']
+
+    def _is_egress_only_policy(self, policy):
+        policy_types = policy['spec'].get('policyTypes', [])
+        return (policy_types == ['Egress'] or
+                (policy['spec'].get('egress') and
+                    not policy['spec'].get('ingress')))
