@@ -13,8 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutronclient.common import exceptions as n_exc
-from oslo_cache import core as cache
+from openstack import exceptions as os_exc
 from oslo_config import cfg as oslo_cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -31,22 +30,6 @@ from kuryr_kubernetes import utils
 LOG = logging.getLogger(__name__)
 
 
-vif_handler_caching_opts = [
-    oslo_cfg.BoolOpt('caching', default=True),
-    oslo_cfg.IntOpt('cache_time', default=120),
-]
-
-oslo_cfg.CONF.register_opts(vif_handler_caching_opts,
-                            "vif_handler_caching")
-
-cache.configure(oslo_cfg.CONF)
-vif_handler_cache_region = cache.create_region()
-MEMOIZE = cache.get_memoization_decorator(
-    oslo_cfg.CONF, vif_handler_cache_region, "vif_handler_caching")
-
-cache.configure_cache_region(oslo_cfg.CONF, vif_handler_cache_region)
-
-
 class VIFHandler(k8s_base.ResourceEventHandler):
     """Controller side of VIF binding process for Kubernetes pods.
 
@@ -80,6 +63,23 @@ class VIFHandler(k8s_base.ResourceEventHandler):
                 drivers.ServiceSecurityGroupsDriver.get_instance())
 
     def on_present(self, pod):
+        state = driver_utils.get_pod_state(pod)
+        if (self._is_pod_completed(pod)):
+            if state:
+                LOG.debug("Pod has completed execution, removing the vifs")
+                self.on_deleted(pod)
+                try:
+                    self._set_pod_state(pod, None)
+                except k_exc.K8sClientException:
+                    LOG.exception("Could not clear pod annotation")
+                    raise k_exc.ResourceNotReady(pod['metadata']['name'])
+                except k_exc.K8sResourceNotFound:
+                    pass
+            else:
+                LOG.debug("Pod has completed execution, no annotation found."
+                          " Skipping")
+            return
+
         if (driver_utils.is_host_network(pod) or
                 not self._is_pod_scheduled(pod)):
             # REVISIT(ivc): consider an additional configurable check that
@@ -87,14 +87,13 @@ class VIFHandler(k8s_base.ResourceEventHandler):
             # where certain pods/namespaces/nodes can be managed by other
             # networking solutions/CNI drivers.
             return
-        state = driver_utils.get_pod_state(pod)
         LOG.debug("Got VIFs from annotation: %r", state)
         project_id = self._drv_project.get_project(pod)
         security_groups = self._drv_sg.get_security_groups(pod, project_id)
         if not state:
             try:
                 subnets = self._drv_subnets.get_subnets(pod, project_id)
-            except (n_exc.NotFound, k_exc.K8sResourceNotFound):
+            except (os_exc.ResourceNotFound, k_exc.K8sResourceNotFound):
                 LOG.warning("Subnet does not exists. If namespace driver is "
                             "used, probably the namespace for the pod is "
                             "already deleted. So this pod does not need to "
@@ -123,7 +122,8 @@ class VIFHandler(k8s_base.ResourceEventHandler):
             if additional_vifs:
                 state.additional_vifs = {}
                 for i, vif in enumerate(additional_vifs, start=1):
-                    k = constants.ADDITIONAL_IFNAME_PREFIX + str(i)
+                    k = (oslo_cfg.CONF.kubernetes.additional_ifname_prefix
+                         + str(i))
                     state.additional_vifs[k] = vif
 
             try:
@@ -140,13 +140,14 @@ class VIFHandler(k8s_base.ResourceEventHandler):
             changed = False
             try:
                 for ifname, vif in state.vifs.items():
-                    if vif.plugin == constants.KURYR_VIF_TYPE_SRIOV:
+                    if (vif.plugin == constants.KURYR_VIF_TYPE_SRIOV and
+                            oslo_cfg.CONF.sriov.enable_node_annotations):
                         driver_utils.update_port_pci_info(pod, vif)
                     if not vif.active:
                         try:
                             self._drv_vif_pool.activate_vif(pod, vif)
                             changed = True
-                        except n_exc.PortNotFoundClient:
+                        except os_exc.ResourceNotFound:
                             LOG.debug("Port not found, possibly already "
                                       "deleted. No need to activate it")
             finally:
@@ -193,10 +194,10 @@ class VIFHandler(k8s_base.ResourceEventHandler):
         except k_exc.ResourceNotReady:
             # NOTE(ltomasbo): If the namespace object gets deleted first the
             # namespace security group driver will raise a ResourceNotReady
-            # exception as it cannot access anymore the kuryrnet CRD annotated
-            # on the namespace object. In such case we set security groups to
-            # empty list so that if pools are enabled they will be properly
-            # released.
+            # exception as it cannot access anymore the kuryrnetwork CRD
+            # annotated on the namespace object. In such case we set security
+            # groups to empty list so that if pools are enabled they will be
+            # properly released.
             security_groups = []
 
         state = driver_utils.get_pod_state(pod)
@@ -210,13 +211,9 @@ class VIFHandler(k8s_base.ResourceEventHandler):
             services = driver_utils.get_services()
             self._update_services(services, crd_pod_selectors, project_id)
 
-    @MEMOIZE
     def is_ready(self, quota):
-        neutron = clients.get_neutron_client()
-        port_quota = quota['port']
-        port_func = neutron.list_ports
-        if utils.has_limit(port_quota):
-            return utils.is_available('ports', port_quota, port_func)
+        if utils.has_limit(quota.ports):
+            return utils.is_available('ports', quota.ports)
         return True
 
     @staticmethod
@@ -228,12 +225,23 @@ class VIFHandler(k8s_base.ResourceEventHandler):
         except KeyError:
             return False
 
+    @staticmethod
+    def _is_pod_completed(pod):
+        try:
+            return (pod['status']['phase'] in
+                    (constants.K8S_POD_STATUS_SUCCEEDED,
+                     constants.K8S_POD_STATUS_FAILED))
+        except KeyError:
+            return False
+
     def _set_pod_state(self, pod, state):
         # TODO(ivc): extract annotation interactions
         if not state:
+            old_annotation = pod['metadata'].get('annotations', {})
             LOG.debug("Removing VIFs annotation: %r for pod %s/%s (uid: %s)",
-                      state, pod['metadata']['namespace'],
-                      pod['metadata']['name'], pod['metadata']['uid'])
+                      old_annotation.get(constants.K8S_ANNOTATION_VIF),
+                      pod['metadata']['namespace'], pod['metadata']['name'],
+                      pod['metadata']['uid'])
             annotation = None
         else:
             state_dict = state.obj_to_primitive()
