@@ -12,15 +12,21 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import mock
+import os
+from unittest import mock
 import uuid
 
+
 from os_vif import objects as osv_objects
+from os_vif.objects import fields as osv_fields
 from oslo_config import cfg
+from oslo_utils import uuidutils
 
 from kuryr_kubernetes.cni.binding import base
 from kuryr_kubernetes.cni.binding import sriov
+from kuryr_kubernetes.cni.binding import vhostuser
 from kuryr_kubernetes import constants as k_const
+from kuryr_kubernetes import exceptions
 from kuryr_kubernetes import objects
 from kuryr_kubernetes.tests import base as test_base
 from kuryr_kubernetes.tests import fake
@@ -60,6 +66,7 @@ class TestDriverMixin(test_base.TestCase):
                 'bridge': mock.Mock(
                     __enter__=mock.Mock(return_value=self.m_bridge_iface),
                     __exit__=mock.Mock(return_value=None),
+                    mtu=1,
                 ),
                 'c_interface': mock.Mock(
                     __enter__=mock.Mock(return_value=self.m_c_iface),
@@ -95,8 +102,13 @@ class TestDriverMixin(test_base.TestCase):
         if report:
             report.assert_called_once()
 
+    @mock.patch('kuryr_kubernetes.cni.binding.base.get_ipdb')
     @mock.patch('os_vif.unplug')
-    def _test_disconnect(self, m_vif_unplug, report=None):
+    def _test_disconnect(self, m_vif_unplug, m_get_ipdb, report=None):
+        def get_ipdb(netns=None):
+            return self.ipdbs[netns]
+        m_get_ipdb.side_effect = get_ipdb
+
         base.disconnect(self.vif, self.instance_info, self.ifname, self.netns,
                         report)
         m_vif_unplug.assert_called_once_with(self.vif, self.instance_info)
@@ -179,14 +191,18 @@ class TestNestedVlanDriver(TestDriverMixin, test_base.TestCase):
     def test_connect(self):
         self._test_connect()
 
-        self.assertEqual(1, self.h_ipdb_exit.call_count)
-        self.assertEqual(2, self.c_ipdb_exit.call_count)
+        self.assertEqual(2, self.h_ipdb_exit.call_count)
+        self.assertEqual(3, self.c_ipdb_exit.call_count)
 
         self.assertEqual(self.ifname, self.m_h_iface.ifname)
         self.assertEqual(1, self.m_h_iface.mtu)
         self.assertEqual(str(self.vif.address), self.m_h_iface.address)
         self.m_h_iface.up.assert_called_once_with()
 
+    def test_connect_mtu_mismatch(self):
+        self.vif.network.mtu = 2
+        self.assertRaises(exceptions.CNIBindingFailure, self._test_connect)
+
     def test_disconnect(self):
         self._test_disconnect()
 
@@ -201,14 +217,18 @@ class TestNestedMacvlanDriver(TestDriverMixin, test_base.TestCase):
     def test_connect(self):
         self._test_connect()
 
-        self.assertEqual(1, self.h_ipdb_exit.call_count)
-        self.assertEqual(2, self.c_ipdb_exit.call_count)
+        self.assertEqual(2, self.h_ipdb_exit.call_count)
+        self.assertEqual(3, self.c_ipdb_exit.call_count)
 
         self.assertEqual(self.ifname, self.m_h_iface.ifname)
         self.assertEqual(1, self.m_h_iface.mtu)
         self.assertEqual(str(self.vif.address), self.m_h_iface.address)
         self.m_h_iface.up.assert_called_once_with()
 
+    def test_connect_mtu_mismatch(self):
+        self.vif.network.mtu = 2
+        self.assertRaises(exceptions.CNIBindingFailure, self._test_connect)
+
     def test_disconnect(self):
         self._test_disconnect()
 
@@ -218,7 +238,7 @@ class TestSriovDriver(TestDriverMixin, test_base.TestCase):
         super(TestSriovDriver, self).setUp()
         self.vif = fake._fake_vif(objects.vif.VIFSriov)
         self.vif.physnet = 'physnet2'
-        self.pci_info = mock.Mock()
+        self.pci_info = mock.MagicMock()
         self.vif.pod_link = 'pod_link'
         self.vif.pod_name = 'pod_1'
         self.pci = mock.Mock()
@@ -247,6 +267,8 @@ class TestSriovDriver(TestDriverMixin, test_base.TestCase):
                           group='sriov')
         CONF.set_override('resource_driver_mappings', 'sriov:igbvf',
                           group='sriov')
+        CONF.set_override('enable_node_annotations', True,
+                          group='sriov')
 
     @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
                 '_save_pci_info')
@@ -258,6 +280,18 @@ class TestSriovDriver(TestDriverMixin, test_base.TestCase):
         self._test_connect()
         m_save_pci.assert_called_once_with(self.vif.id, self.pci_info)
 
+    @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
+                '_save_pci_info')
+    @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
+                '_process_vif')
+    def test_connect_no_annotations(self, m_proc_vif, m_save_pci):
+        m_proc_vif.return_value = self.pci_info
+        CONF.set_override('enable_node_annotations', False,
+                          group='sriov')
+
+        self._test_connect()
+        m_save_pci.assert_not_called()
+
     @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
                 '_return_device_driver')
     @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
@@ -266,6 +300,19 @@ class TestSriovDriver(TestDriverMixin, test_base.TestCase):
         m_remove_pci.return_value = None
         m_return_device.return_value = None
         self._test_disconnect()
+        m_remove_pci.assert_called_once_with(self.vif.id)
+
+    @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
+                '_return_device_driver')
+    @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
+                '_remove_pci_info')
+    def test_disconnect_no_annotations(self, m_remove_pci, m_return_device):
+        m_return_device.return_value = None
+        m_remove_pci.return_value = None
+        CONF.set_override('enable_node_annotations', False,
+                          group='sriov')
+        self._test_disconnect()
+        m_remove_pci.assert_not_called()
 
     @mock.patch('kuryr_kubernetes.clients.get_pod_resources_client')
     @mock.patch('kuryr_kubernetes.cni.binding.sriov.VIFSriovDriver.'
@@ -387,3 +434,92 @@ class TestSriovDriver(TestDriverMixin, test_base.TestCase):
         cls._return_device_driver(m_driver, self.vif)
         m_driver._bind_device.assert_called_once_with(pci, old_driver,
                                                       new_driver)
+
+
+class TestVHostUserDriver(TestDriverMixin, test_base.TestCase):
+    def setUp(self):
+        super(TestVHostUserDriver, self).setUp()
+        self.vu_mount_point = '/var/run/cni'
+        self.vu_ovs_path = '/var/run/openvswitch'
+        CONF.set_override('mount_point', self.vu_mount_point,
+                          group='vhostuser')
+        CONF.set_override('ovs_vhu_path', self.vu_ovs_path,
+                          group='vhostuser')
+        self.vif = fake._fake_vif(osv_objects.vif.VIFVHostUser)
+        self.vif.path = self.vu_mount_point
+        self.vif.address = '64:0f:2b:5f:0c:1c'
+        self.port_name = vhostuser._get_vhostuser_port_name(self.vif)
+        self.cont_id = uuidutils.generate_uuid()
+
+    @mock.patch('kuryr_kubernetes.cni.binding.base._need_configure_l3')
+    @mock.patch('kuryr_kubernetes.cni.plugins.k8s_cni_registry.'
+                'K8sCNIRegistryPlugin.report_drivers_health')
+    @mock.patch('os.rename')
+    @mock.patch('os.path.exists', mock.Mock(return_value=True))
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser.VIFVHostUserDriver.'
+                '_write_config')
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser._check_sock_file')
+    @mock.patch('os_vif.plug')
+    def test_connect_client(self, m_vif_plug, m_check_sock, m_write_conf,
+                            m_os_rename, m_report, m_need_l3):
+        m_need_l3.return_value = False
+        self.vif.mode = osv_fields.VIFVHostUserMode.CLIENT
+        m_check_sock.return_value = True
+        base.connect(self.vif, self.instance_info, self.ifname, self.netns,
+                     m_report, container_id=self.cont_id)
+        vu_dst_socket = os.path.join(self.vu_mount_point, self.port_name)
+        vu_src_socket = os.path.join(self.vu_ovs_path, self.port_name)
+
+        m_vif_plug.assert_called_once_with(self.vif, self.instance_info)
+        m_os_rename.assert_called_once_with(vu_src_socket, vu_dst_socket)
+        m_write_conf.assert_called_once_with(self.cont_id, self.ifname,
+                                             self.port_name, self.vif)
+        m_report.assert_called_once()
+
+    @mock.patch('kuryr_kubernetes.cni.binding.base._need_configure_l3')
+    @mock.patch('kuryr_kubernetes.cni.plugins.k8s_cni_registry.'
+                'K8sCNIRegistryPlugin.report_drivers_health')
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser.VIFVHostUserDriver.'
+                '_write_config')
+    @mock.patch('os_vif.plug')
+    def test_connect_server(self, m_vif_plug, m_write_conf,
+                            m_report, m_need_l3):
+        m_need_l3.return_value = False
+        self.vif.mode = osv_fields.VIFVHostUserMode.SERVER
+        base.connect(self.vif, self.instance_info, self.ifname, self.netns,
+                     m_report, container_id=self.cont_id)
+        m_vif_plug.assert_called_once_with(self.vif, self.instance_info)
+        m_write_conf.assert_called_once_with(self.cont_id, self.ifname,
+                                             self.port_name, self.vif)
+        m_report.assert_called_once()
+
+    @mock.patch('kuryr_kubernetes.cni.plugins.k8s_cni_registry.'
+                'K8sCNIRegistryPlugin.report_drivers_health')
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser._check_sock_file',
+                mock.Mock(return_value=False))
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser.VIFVHostUserDriver.'
+                '_write_config', mock.Mock())
+    @mock.patch('os_vif.plug')
+    def test_connect_nosocket(self, m_vif_plug, m_report):
+        self.vif.mode = osv_fields.VIFVHostUserMode.CLIENT
+        self.assertRaises(exceptions.CNIError, base.connect, self.vif,
+                          self.instance_info, self.ifname, self.netns,
+                          m_report, container_id=self.cont_id)
+
+    @mock.patch('kuryr_kubernetes.cni.plugins.k8s_cni_registry.'
+                'K8sCNIRegistryPlugin.report_drivers_health')
+    @mock.patch('kuryr_kubernetes.cni.binding.vhostuser._get_vhu_sock')
+    @mock.patch('os.remove')
+    @mock.patch('os.path.exists', mock.Mock(return_value=True))
+    @mock.patch('os_vif.unplug')
+    def test_disconnect(self, m_os_unplug, m_os_remove, m_get_vhu_sock,
+                        m_report):
+        m_get_vhu_sock.return_value = self.port_name
+        base.disconnect(self.vif, self.instance_info, self.ifname, self.netns,
+                        m_report, container_id=self.cont_id)
+        conf_file_path = '{}/{}-{}'.format(self.vu_mount_point,
+                                           self.cont_id, self.ifname)
+        vhu_sock_path = '{}/{}'.format(self.vu_mount_point,
+                                       self.port_name)
+        os_remove_calls = [mock.call(vhu_sock_path), mock.call(conf_file_path)]
+        m_os_remove.assert_has_calls(os_remove_calls)
