@@ -13,13 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-import requests
+import munch
+from unittest import mock
 
-from neutronclient.common import exceptions as n_exc
-from openstack import exceptions as o_exc
-from openstack.load_balancer.v2 import l7_policy as o_l7p
-from openstack.load_balancer.v2 import l7_rule as o_l7r
+from openstack import exceptions as os_exc
 from openstack.load_balancer.v2 import listener as o_lis
 from openstack.load_balancer.v2 import load_balancer as o_lb
 from openstack.load_balancer.v2 import member as o_mem
@@ -68,7 +65,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
                         group='neutron_defaults')
         d = d_lbaasv2.LBaaSv2Driver()
         req = {}
-        d._add_tags('loadbalancer', req)
+        d.add_tags('loadbalancer', req)
         self.assertEqual({'tags': ['foo']}, req)
 
     @mock.patch('kuryr_kubernetes.controller.drivers.lbaasv2.LBaaSv2Driver.'
@@ -76,7 +73,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_add_tags_no_tag(self, _m_get):
         d = d_lbaasv2.LBaaSv2Driver()
         req = {}
-        d._add_tags('loadbalancer', req)
+        d.add_tags('loadbalancer', req)
         self.assertEqual({}, req)
 
     @mock.patch('kuryr_kubernetes.controller.drivers.lbaasv2.LBaaSv2Driver.'
@@ -86,9 +83,9 @@ class TestLBaaSv2Driver(test_base.TestCase):
         self.addCleanup(CONF.clear_override, 'resource_tags',
                         group='neutron_defaults')
         d = d_lbaasv2.LBaaSv2Driver()
-        for res in ('loadbalancer', 'listener', 'pool', 'l7policy'):
+        for res in ('loadbalancer', 'listener', 'pool'):
             req = {}
-            d._add_tags(res, req)
+            d.add_tags(res, req)
             self.assertEqual({'description': 'foo'}, req,
                              'No description added to resource %s' % res)
 
@@ -101,7 +98,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         d = d_lbaasv2.LBaaSv2Driver()
         for res in ('member', 'rule'):
             req = {}
-            d._add_tags(res, req)
+            d.add_tags(res, req)
             self.assertEqual({}, req, 'Unnecessary description added to '
                                       'resource %s' % res)
 
@@ -109,11 +106,10 @@ class TestLBaaSv2Driver(test_base.TestCase):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         lbaas.get_all_version_data.return_value = OCTAVIA_VERSIONS
         self.assertEqual((2, 2),
-                         d_lbaasv2.LBaaSv2Driver.get_octavia_version(
-                             d_lbaasv2.LBaaSv2Driver()))
+                         d_lbaasv2.LBaaSv2Driver.get_octavia_version(None))
 
     def test_ensure_loadbalancer(self):
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
         expected_resp = obj_lbaas.LBaaSLoadBalancer(
@@ -126,7 +122,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         lb_name = 'just_a_name'
 
         m_driver._ensure.return_value = expected_resp
-        neutron.update_port = mock.Mock()
+        os_net.update_port = mock.Mock()
         resp = cls.ensure_loadbalancer(m_driver, lb_name, project_id,
                                        subnet_id, ip, sg_ids, 'ClusterIP')
         m_driver._ensure.assert_called_once_with(mock.ANY,
@@ -138,7 +134,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         self.assertEqual(subnet_id, req.subnet_id)
         self.assertEqual(ip, str(req.ip))
         self.assertEqual(expected_resp, resp)
-        neutron.update_port.assert_not_called()
+        os_net.update_port.assert_not_called()
 
     def test_ensure_loadbalancer_not_ready(self):
         cls = d_lbaasv2.LBaaSv2Driver
@@ -156,7 +152,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
                           sg_ids, 'ClusterIP')
 
     def test_cascade_release_loadbalancer(self):
-        self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         lbaas.lbaas_loadbalancer_path = "boo %s"
         cls = d_lbaasv2.LBaaSv2Driver
@@ -217,21 +213,20 @@ class TestLBaaSv2Driver(test_base.TestCase):
         loadbalancer = obj_lbaas.LBaaSLoadBalancer(
             id=loadbalancer_id, name=name, project_id=project_id,
             subnet_id=subnet_id, ip=ip, provider=provider)
-        m_driver._ensure_provisioned.side_effect = o_exc.BadRequestException
+        m_driver._ensure_provisioned.side_effect = os_exc.BadRequestException
 
         resp = cls.ensure_listener(m_driver, loadbalancer,
                                    protocol, port)
         self.assertIsNone(resp)
 
     def test_release_listener(self):
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        neutron.list_security_group_rules.return_value = {
-            'security_group_rules': []}
+        os_net.security_group_rules.return_value = (x for x in [])
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        m_driver._get_vip_port.return_value = {
-            'security_groups': [mock.sentinel.sg_id]}
+        m_driver._get_vip_port.return_value = munch.Munch({
+            'security_group_ids': [mock.sentinel.sg_id]})
         loadbalancer = mock.Mock()
         listener = mock.Mock()
 
@@ -328,6 +323,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_loadbalancer(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         loadbalancer = obj_lbaas.LBaaSLoadBalancer(
             name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
             subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
@@ -340,12 +336,12 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'vip_subnet_id': loadbalancer.subnet_id,
         }
         resp = o_lb.LoadBalancer(id=loadbalancer_id, provider='haproxy')
-        m_driver._post_lb_resource.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+        lbaas.create_load_balancer.return_value = resp
+        m_driver._get_vip_port.return_value = munch.Munch(
+            {'id': mock.sentinel.port_id})
 
         ret = cls._create_loadbalancer(m_driver, loadbalancer)
-        m_driver._post_lb_resource.assert_called_once_with(o_lb.LoadBalancer,
-                                                           req)
+        lbaas.create_load_balancer.assert_called_once_with(**req)
         for attr in loadbalancer.obj_fields:
             self.assertEqual(getattr(loadbalancer, attr),
                              getattr(ret, attr))
@@ -354,6 +350,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_loadbalancer_provider_defined(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         loadbalancer = obj_lbaas.LBaaSLoadBalancer(
             name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
             subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
@@ -368,12 +365,12 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'provider': loadbalancer.provider,
         }
         resp = o_lb.LoadBalancer(id=loadbalancer_id, provider='amphora')
-        m_driver._post_lb_resource.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+        lbaas.create_load_balancer.return_value = resp
+        m_driver._get_vip_port.return_value = munch.Munch(
+            {'id': mock.sentinel.port_id})
 
         ret = cls._create_loadbalancer(m_driver, loadbalancer)
-        m_driver._post_lb_resource.assert_called_once_with(o_lb.LoadBalancer,
-                                                           req)
+        lbaas.create_load_balancer.assert_called_once_with(**req)
         for attr in loadbalancer.obj_fields:
             self.assertEqual(getattr(loadbalancer, attr),
                              getattr(ret, attr))
@@ -382,6 +379,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_loadbalancer_provider_mismatch(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         loadbalancer = obj_lbaas.LBaaSLoadBalancer(
             name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
             subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
@@ -396,12 +394,12 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'provider': loadbalancer.provider,
         }
         resp = o_lb.LoadBalancer(id=loadbalancer_id, provider='haproxy')
-        m_driver._post_lb_resource.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+        lbaas.create_load_balancer.return_value = resp
+        m_driver._get_vip_port.return_value = munch.Munch(
+            {'id': mock.sentinel.port_id})
 
         ret = cls._create_loadbalancer(m_driver, loadbalancer)
-        m_driver._post_lb_resource.assert_called_once_with(o_lb.LoadBalancer,
-                                                           req)
+        lbaas.create_load_balancer.assert_called_once_with(**req)
         self.assertIsNone(ret)
 
     def test_find_loadbalancer(self):
@@ -416,7 +414,8 @@ class TestLBaaSv2Driver(test_base.TestCase):
         resp = iter([o_lb.LoadBalancer(id=loadbalancer_id, provider='haproxy',
                                        provisioning_status='ACTIVE')])
         lbaas.load_balancers.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+        m_driver._get_vip_port.return_value = munch.Munch(
+            {'id': mock.sentinel.port_id})
 
         ret = cls._find_loadbalancer(m_driver, loadbalancer)
         lbaas.load_balancers.assert_called_once_with(
@@ -460,7 +459,8 @@ class TestLBaaSv2Driver(test_base.TestCase):
         resp = iter([o_lb.LoadBalancer(id=loadbalancer_id, provider='haproxy',
                                        provisioning_status='ERROR')])
         lbaas.load_balancers.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+        m_driver._get_vip_port.return_value = munch.Munch(
+            {'id': mock.sentinel.port_id})
 
         ret = cls._find_loadbalancer(m_driver, loadbalancer)
         lbaas.load_balancers.assert_called_once_with(
@@ -474,6 +474,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_listener(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         listener = obj_lbaas.LBaaSListener(
             name='TEST_NAME', project_id='TEST_PROJECT', protocol='TCP',
             port=1234, loadbalancer_id='00EE9E11-91C2-41CF-8FD4-7970579E5C4C')
@@ -485,10 +486,10 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'protocol': listener.protocol,
             'protocol_port': listener.port}
         resp = o_lis.Listener(id=listener_id)
-        m_driver._post_lb_resource.return_value = resp
+        lbaas.create_listener.return_value = resp
 
         ret = cls._create_listener(m_driver, listener)
-        m_driver._post_lb_resource.assert_called_once_with(o_lis.Listener, req)
+        lbaas.create_listener.assert_called_once_with(**req)
         for attr in listener.obj_fields:
             self.assertEqual(getattr(listener, attr),
                              getattr(ret, attr))
@@ -538,6 +539,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_pool(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         lb_algorithm = 'ROUND_ROBIN'
         pool = obj_lbaas.LBaaSPool(
             name='TEST_NAME', project_id='TEST_PROJECT', protocol='TCP',
@@ -552,10 +554,41 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'protocol': pool.protocol,
             'lb_algorithm': lb_algorithm}
         resp = o_pool.Pool(id=pool_id)
-        m_driver._post_lb_resource.return_value = resp
+        lbaas.create_pool.return_value = resp
 
         ret = cls._create_pool(m_driver, pool)
-        m_driver._post_lb_resource.assert_called_once_with(o_pool.Pool, req)
+        lbaas.create_pool.assert_called_once_with(**req)
+        for attr in pool.obj_fields:
+            self.assertEqual(getattr(pool, attr),
+                             getattr(ret, attr))
+        self.assertEqual(pool_id, ret.id)
+
+    def test_create_pool_with_different_lb_algorithm(self):
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        lb_algorithm = 'SOURCE_IP_PORT'
+        pool = obj_lbaas.LBaaSPool(
+            name='TEST_NAME', project_id='TEST_PROJECT', protocol='TCP',
+            listener_id='A57B7771-6050-4CA8-A63C-443493EC98AB',
+            loadbalancer_id='00EE9E11-91C2-41CF-8FD4-7970579E5C4C')
+        pool_id = 'D4F35594-27EB-4F4C-930C-31DD40F53B77'
+        req = {
+            'name': pool.name,
+            'project_id': pool.project_id,
+            'listener_id': pool.listener_id,
+            'loadbalancer_id': pool.loadbalancer_id,
+            'protocol': pool.protocol,
+            'lb_algorithm': lb_algorithm}
+        resp = o_pool.Pool(id=pool_id)
+        lbaas.create_pool.return_value = resp
+        CONF.set_override('lb_algorithm', lb_algorithm,
+                          group='octavia_defaults')
+        self.addCleanup(CONF.clear_override, 'lb_algorithm',
+                        group='octavia_defaults')
+
+        ret = cls._create_pool(m_driver, pool)
+        lbaas.create_pool.assert_called_once_with(**req)
         for attr in pool.obj_fields:
             self.assertEqual(getattr(pool, attr),
                              getattr(ret, attr))
@@ -564,6 +597,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_pool_conflict(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         lb_algorithm = 'ROUND_ROBIN'
         pool = obj_lbaas.LBaaSPool(
             name='TEST_NAME', project_id='TEST_PROJECT', protocol='TCP',
@@ -576,11 +610,11 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'loadbalancer_id': pool.loadbalancer_id,
             'protocol': pool.protocol,
             'lb_algorithm': lb_algorithm}
-        m_driver._post_lb_resource.side_effect = n_exc.StateInvalidClient
+        lbaas.create_pool.side_effect = os_exc.BadRequestException
 
-        self.assertRaises(n_exc.StateInvalidClient, cls._create_pool, m_driver,
-                          pool)
-        m_driver._post_lb_resource.assert_called_once_with(o_pool.Pool, req)
+        self.assertRaises(os_exc.BadRequestException, cls._create_pool,
+                          m_driver, pool)
+        lbaas.create_pool.assert_called_once_with(**req)
 
     def test_find_pool_by_listener(self):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
@@ -628,6 +662,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_create_member(self):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         member = obj_lbaas.LBaaSMember(
             name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
             port=1234, subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
@@ -640,11 +675,10 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'address': str(member.ip),
             'protocol_port': member.port}
         resp = o_mem.Member(id=member_id)
-        m_driver._post_lb_resource.return_value = resp
+        lbaas.create_member.return_value = resp
 
         ret = cls._create_member(m_driver, member)
-        m_driver._post_lb_resource.assert_called_once_with(
-            o_mem.Member, req, pool_id=member.pool_id)
+        lbaas.create_member.assert_called_once_with(member.pool_id, **req)
         for attr in member.obj_fields:
             self.assertEqual(getattr(member, attr),
                              getattr(ret, attr))
@@ -726,20 +760,11 @@ class TestLBaaSv2Driver(test_base.TestCase):
 
     def test_ensure_with_conflict(self):
         self._verify_ensure_with_exception(
-            o_exc.ConflictException(http_status=409))
+            os_exc.ConflictException(http_status=409))
 
     def test_ensure_with_internalservererror(self):
         self._verify_ensure_with_exception(
-            o_exc.HttpException(http_status=500))
-
-    def test_ensure_with_httperrors(self):
-        resp = requests.Response()
-        resp.status_code = 409
-        self._verify_ensure_with_exception(
-            requests.exceptions.HTTPError(response=resp))
-        resp.status_code = 500
-        self._verify_ensure_with_exception(
-            requests.exceptions.HTTPError(response=resp))
+            os_exc.HttpException(http_status=500))
 
     def test_request(self):
         cls = d_lbaasv2.LBaaSv2Driver
@@ -750,12 +775,9 @@ class TestLBaaSv2Driver(test_base.TestCase):
         find = mock.sentinel.find
         timer = [mock.sentinel.t0]
         m_driver._provisioning_timer.return_value = timer
-        resp = requests.Response()
-        resp.status_code = 400
-        m_driver._ensure.side_effect = requests.exceptions.HTTPError(
-            response=resp)
+        m_driver._ensure.side_effect = os_exc.BadRequestException()
 
-        self.assertRaises(requests.exceptions.HTTPError,
+        self.assertRaises(os_exc.BadRequestException,
                           cls._ensure_provisioned, m_driver,
                           loadbalancer, obj, create, find)
 
@@ -808,7 +830,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         m_delete = mock.Mock()
         timer = [mock.sentinel.t0, mock.sentinel.t1]
         m_driver._provisioning_timer.return_value = timer
-        m_delete.side_effect = [o_exc.BadRequestException, None]
+        m_delete.side_effect = [os_exc.BadRequestException, None]
 
         cls._release(m_driver, loadbalancer, obj, m_delete)
 
@@ -824,7 +846,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         m_delete = mock.Mock()
         timer = [mock.sentinel.t0, mock.sentinel.t1]
         m_driver._provisioning_timer.return_value = timer
-        m_delete.side_effect = o_exc.NotFoundException
+        m_delete.side_effect = os_exc.NotFoundException
 
         cls._release(m_driver, loadbalancer, obj, m_delete)
 
@@ -839,7 +861,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
         m_delete = mock.Mock()
         timer = [mock.sentinel.t0, mock.sentinel.t1]
         m_driver._provisioning_timer.return_value = timer
-        m_delete.side_effect = o_exc.ConflictException
+        m_delete.side_effect = os_exc.ConflictException
 
         self.assertRaises(k_exc.ResourceNotReady, cls._release, m_driver,
                           loadbalancer, obj, m_delete)
@@ -883,363 +905,3 @@ class TestLBaaSv2Driver(test_base.TestCase):
     def test_provisioning_timer(self):
         # REVISIT(ivc): add test if _provisioning_timer is to stay
         self.skipTest("not implemented")
-
-    def test_get_pool_by_name_not_found(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        pools = o_pool.Pool(name='KUKU',
-                            id='a2a62ea7-e3bf-40df-8c09-aa0c29876a6b')
-        lbaas.pools.return_value = [pools]
-        pool_name = 'NOT_KUKU'
-        project_id = 'TEST_PROJECT'
-
-        pool_id = cls.get_pool_by_name(m_driver, pool_name, project_id)
-        self.assertIsNone(pool_id)
-
-    def test_get_pool_by_name_found(self):
-        self._test_get_pool_by_name_found(listener_is_empty=False)
-
-    def test_get_pool_by_name_found_listener_is_empty(self):
-        self._test_get_pool_by_name_found(listener_is_empty=True)
-
-    def _test_get_pool_by_name_found(self, listener_is_empty):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        pool_name = 'KUKU'
-        pool_lb_id = "607226db-27ef-4d41-ae89-f2a800e9c2db"
-        pool_project_id = "e3cd678b11784734bc366148aa37580e"
-        pool_id = "ddb2b28f-89e9-45d3-a329-a359c3e39e4a"
-        pool_protocol = "HTTP"
-        pool_listener_id = "023f2e34-7806-443b-bfae-16c324569a3d"
-
-        if listener_is_empty:
-            resp_listeners = []
-        else:
-            resp_listeners = [o_lis.Listener(id=pool_listener_id)]
-
-        listener_id = (resp_listeners[0].id if resp_listeners else None)
-        expected_result = obj_lbaas.LBaaSPool(
-            name=pool_name, project_id=pool_project_id,
-            loadbalancer_id=pool_lb_id,
-            listener_id=listener_id,
-            protocol=pool_protocol,
-            id=pool_id)
-
-        resp = [o_pool.Pool(
-            protocol=pool_protocol,
-            loadbalancers=[o_lb.LoadBalancer(id=pool_lb_id)],
-            listeners=resp_listeners,
-            project_id=pool_project_id,
-            id=pool_id,
-            name=pool_name,
-        )]
-
-        lbaas.pools.return_value = resp
-
-        pool = cls.get_pool_by_name(m_driver, pool_name, pool_project_id)
-        lbaas.pools.assert_called_once()
-        for attr in expected_result.obj_fields:
-            self.assertEqual(getattr(expected_result, attr),
-                             getattr(pool, attr))
-
-    def test_get_pool_by_name_empty_list(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        pools = {}
-        lbaas.pools.return_value = [pools]
-        pool_name = 'NOT_KUKU'
-        project_id = 'TEST_PROJECT'
-
-        pool = cls.get_pool_by_name(m_driver, pool_name, project_id)
-        self.assertIsNone(pool)
-
-    def test_get_lb_by_uuid(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
-        loadbalancer_vip = '1.2.3.4'
-        loadbalancer_vip_port_id = '00EE9E11-91C2-41CF-8FD4-7970579EFFFF'
-        loadbalancer_project_id = '00EE9E11-91C2-41CF-8FD4-7970579EAAAA'
-        loadbalancer_name = 'MyName'
-        loadbalancer_subnet_id = '00EE9E11-91C2-41CF-8FD4-7970579EBBBB'
-        loadbalancer_provider = 'haproxy'
-
-        expected_lb = obj_lbaas.LBaaSLoadBalancer(
-            id=loadbalancer_id, port_id=loadbalancer_vip_port_id,
-            name=loadbalancer_name, project_id=loadbalancer_project_id,
-            subnet_id=loadbalancer_subnet_id, ip=loadbalancer_vip,
-            security_groups=None, provider=loadbalancer_provider)
-
-        resp = o_lb.LoadBalancer(id=loadbalancer_id,
-                                 vip_port_id=loadbalancer_vip_port_id,
-                                 name=loadbalancer_name,
-                                 project_id=loadbalancer_project_id,
-                                 vip_subnet_id=loadbalancer_subnet_id,
-                                 vip_address=loadbalancer_vip,
-                                 provider=loadbalancer_provider)
-
-        lbaas.get_load_balancer.return_value = resp
-
-        ret = cls.get_lb_by_uuid(m_driver, loadbalancer_id)
-        lbaas.get_load_balancer.assert_called_once()
-        for attr in expected_lb.obj_fields:
-            self.assertEqual(getattr(expected_lb, attr),
-                             getattr(ret, attr))
-        self.assertEqual(loadbalancer_id, ret.id)
-
-    def test_get_lb_by_uuid_not_found(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        resp = {'loadbalancer': {}}
-        lbaas.get_load_balancer.return_value = resp
-
-        requested_uuid = '00EE9E11-91C2-41CF-8FD4-7970579EFFFF'
-        lbaas.get_load_balancer.side_effect = o_exc.ResourceNotFound
-
-        ret = cls.get_lb_by_uuid(m_driver, requested_uuid)
-        lbaas.get_load_balancer.assert_called_once()
-        self.assertIsNone(ret)
-
-    def test_ensure_l7policy(self):
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        expected_resp = mock.sentinel.expected_resp
-        loadbalancer = mock.sentinel.expected_resp
-        route_name = 'ROUTE_NAME'
-        namespace = 'NAMESPACE'
-        listener_id = 'D4F35594-27EB-4F4C-930C-31DD40F53B77'
-        pool = obj_lbaas.LBaaSPool(
-            id='00EE9E11-91C2-41CF-8FD4-7970579E5C4C',
-            project_id='TEST_PROJECT',
-            name='NAME',
-            loadbalancer_id='010101',
-            listener_id='12345',
-            protocol='TCP'
-        )
-
-        m_driver._ensure_provisioned.return_value = expected_resp
-
-        cls.ensure_l7_policy(
-            m_driver, namespace, route_name, loadbalancer, pool, listener_id)
-
-        m_driver._ensure_provisioned.assert_called_once_with(
-            loadbalancer, mock.ANY, m_driver._create_l7_policy,
-            m_driver._find_l7_policy)
-        l7policy = m_driver._ensure_provisioned.call_args[0][1]
-
-        self.assertEqual("%s%s" % (namespace, route_name), l7policy.name)
-        self.assertEqual(listener_id, l7policy.listener_id)
-        self.assertEqual(pool.id, l7policy.redirect_pool_id)
-        self.assertEqual(pool.project_id, l7policy.project_id)
-
-    def test_release_l7policy(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        loadbalancer = mock.Mock()
-        l7_policy = mock.Mock()
-        cls.release_l7_policy(m_driver, loadbalancer, l7_policy)
-
-        m_driver._release.assert_called_once_with(
-            loadbalancer, l7_policy, lbaas.delete_l7_policy,
-            l7_policy.id)
-
-    def test_create_l7policy(self):
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        l7_policy = obj_lbaas.LBaaSL7Policy(
-            name='TEST_NAME',
-            project_id='TEST_PROJECT',
-            listener_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            redirect_pool_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1')
-
-        l7policy_id = '3A70CEC0-392D-4BC1-A27C-06E63A0FD54F'
-        req = {
-            'action': 'REDIRECT_TO_POOL',
-            'listener_id': l7_policy.listener_id,
-            'name': l7_policy.name,
-            'project_id': l7_policy.project_id,
-            'redirect_pool_id': l7_policy.redirect_pool_id}
-        resp = {'id': l7policy_id}
-        m_driver._post_lb_resource.return_value = resp
-
-        ret = cls._create_l7_policy(m_driver, l7_policy)
-        m_driver._post_lb_resource.assert_called_once_with(o_l7p.L7Policy, req)
-        for attr in l7_policy.obj_fields:
-            self.assertEqual(getattr(l7_policy, attr),
-                             getattr(ret, attr))
-        self.assertEqual(l7policy_id, ret.id)
-
-    def test_find_l7_policy(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        l7_policy = obj_lbaas.LBaaSL7Policy(
-            name='TEST_NAME',
-            project_id='TEST_PROJECT',
-            listener_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            redirect_pool_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1')
-
-        l7policy_id = '3A70CEC0-392D-4BC1-A27C-06E63A0FD54F'
-
-        resp = iter([o_l7p.L7Policy(id=l7policy_id)])
-        lbaas.l7_policies.return_value = resp
-
-        ret = cls._find_l7_policy(m_driver, l7_policy)
-        lbaas.l7_policies.assert_called_once_with(
-            name=l7_policy.name,
-            project_id=l7_policy.project_id,
-            redirect_pool_id=l7_policy.redirect_pool_id,
-            listener_id=l7_policy.listener_id)
-        for attr in l7_policy.obj_fields:
-            self.assertEqual(getattr(l7_policy, attr),
-                             getattr(ret, attr))
-        self.assertEqual(l7policy_id, ret.id)
-
-    def test_find_l7_policy_not_found(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        l7_policy = obj_lbaas.LBaaSL7Policy(
-            name='TEST_NAME',
-            project_id='TEST_PROJECT',
-            listener_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            redirect_pool_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1')
-
-        resp = iter([])
-        lbaas.l7_policies.return_value = resp
-
-        ret = cls._find_l7_policy(m_driver, l7_policy)
-        lbaas.l7_policies.assert_called_once_with(
-            name=l7_policy.name,
-            project_id=l7_policy.project_id,
-            redirect_pool_id=l7_policy.redirect_pool_id,
-            listener_id=l7_policy.listener_id)
-        self.assertIsNone(ret)
-
-    def test_ensure_l7_rule(self):
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        expected_resp = mock.sentinel.expected_resp
-        loadbalancer = mock.sentinel.expected_resp
-        compare_type = 'EQUAL_TO'
-        type = 'HOST_NAME'
-        value = 'www.test.com'
-        l7_policy = obj_lbaas.LBaaSL7Policy(
-            id='00EE9E11-91C2-41CF-8FD4-7970579E5C4C',
-            name='TEST_NAME',
-            project_id='TEST_PROJECT',
-            listener_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            redirect_pool_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1')
-
-        m_driver._ensure_provisioned.return_value = expected_resp
-
-        cls.ensure_l7_rule(
-            m_driver, loadbalancer, l7_policy, compare_type, type, value)
-
-        m_driver._ensure_provisioned.assert_called_once_with(
-            loadbalancer, mock.ANY, m_driver._create_l7_rule,
-            m_driver._find_l7_rule)
-        l7rule = m_driver._ensure_provisioned.call_args[0][1]
-
-        self.assertEqual(compare_type, l7rule.compare_type)
-        self.assertEqual(type, l7rule.type)
-        self.assertEqual(value, l7rule.value)
-
-    def test_release_l7_rule(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        loadbalancer = mock.Mock()
-        l7_rule = mock.Mock()
-        cls.release_l7_rule(m_driver, loadbalancer, l7_rule)
-
-        m_driver._release.assert_called_once_with(
-            loadbalancer, l7_rule, lbaas.delete_l7_rule,
-            l7_rule.id, l7_rule.l7policy_id)
-
-    def test_create_l7_rule(self):
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-
-        l7_rule = obj_lbaas.LBaaSL7Rule(
-            compare_type='EQUAL_TO',
-            l7policy_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            type='HOST_NAME',
-            value='www.test.com')
-
-        l7_rule_id = '3A70CEC0-392D-4BC1-A27C-06E63A0FD54F'
-
-        req = {'compare_type': l7_rule.compare_type,
-               'type': l7_rule.type,
-               'value': l7_rule.value}
-
-        resp = {'id': l7_rule_id}
-        m_driver._post_lb_resource.return_value = resp
-
-        ret = cls._create_l7_rule(m_driver, l7_rule)
-        m_driver._post_lb_resource.assert_called_once_with(
-            o_l7r.L7Rule, req, l7policy_id=l7_rule.l7policy_id)
-        for attr in l7_rule.obj_fields:
-            self.assertEqual(getattr(l7_rule, attr),
-                             getattr(ret, attr))
-        self.assertEqual(l7_rule_id, ret.id)
-
-    def test_find_l7_rule(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        l7_rule = obj_lbaas.LBaaSL7Rule(
-            compare_type='EQUAL_TO',
-            l7policy_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            type='HOST_NAME',
-            value='www.test.com')
-
-        l7_rule_id = '3A70CEC0-392D-4BC1-A27C-06E63A0FD54F'
-        resp = iter([o_l7r.L7Rule(id=l7_rule_id)])
-        lbaas.l7_rules.return_value = resp
-
-        ret = cls._find_l7_rule(m_driver, l7_rule)
-        lbaas.l7_rules.assert_called_once_with(
-            l7_rule.l7policy_id,
-            type=l7_rule.type,
-            value=l7_rule.value,
-            compare_type=l7_rule.compare_type)
-
-        for attr in l7_rule.obj_fields:
-            self.assertEqual(getattr(l7_rule, attr),
-                             getattr(ret, attr))
-        self.assertEqual(l7_rule_id, ret.id)
-
-    def test_find_l7_rule_not_found(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        l7_rule = obj_lbaas.LBaaSL7Rule(
-            compare_type='EQUAL_TO',
-            l7policy_id='D4F35594-27EB-4F4C-930C-31DD40F53B77',
-            type='HOST_NAME',
-            value='www.test.com')
-
-        resp = iter([])
-        lbaas.l7_rules.return_value = resp
-
-        ret = cls._find_l7_rule(m_driver, l7_rule)
-        lbaas.l7_rules.assert_called_once_with(
-            l7_rule.l7policy_id,
-            type=l7_rule.type,
-            value=l7_rule.value,
-            compare_type=l7_rule.compare_type)
-        self.assertIsNone(ret)
