@@ -10,16 +10,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 import threading
 
 from kuryr.lib import utils as lib_utils
-from neutronclient.common import exceptions as n_exc
+from openstack import exceptions as o_exc
 
 from kuryr_kubernetes.controller.drivers import nested_macvlan_vif
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.tests import base as test_base
+from kuryr_kubernetes.tests import fake
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 
 
@@ -31,7 +33,7 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
     def test_request_vif(self, m_to_vif):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -39,51 +41,51 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         security_groups = mock.sentinel.security_groups
         container_mac = mock.sentinel.mac_address
         container_ip = mock.sentinel.ip_address
-        container_port = self._get_fake_port(mac_address=container_mac,
-                                             ip_address=container_ip)
+        container_port = fake.get_port_obj(mac_address=container_mac,
+                                           ip_address=container_ip)
 
         vif = mock.Mock()
-        port_request = mock.sentinel.port_request
-        vm_port = self._get_fake_port()
+        port_request = {'foo': mock.sentinel.port_request}
+        vm_port = fake.get_port_obj()
 
         m_to_vif.return_value = vif
         m_driver._get_port_request.return_value = port_request
         m_driver._get_parent_port.return_value = vm_port
         m_driver._try_update_port.return_value = 0
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
-        neutron.create_port.return_value = container_port
+        os_net.create_port.return_value = container_port
 
         self.assertEqual(vif, cls.request_vif(m_driver, pod, project_id,
                                               subnets, security_groups))
 
         m_driver._get_port_request.assert_called_once_with(
             pod, project_id, subnets, security_groups)
-        neutron.create_port.assert_called_once_with(port_request)
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        os_net.create_port.assert_called_once_with(**port_request)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._try_update_port.assert_called_once()
-        m_to_vif.assert_called_once_with(container_port['port'], subnets)
+        m_to_vif.assert_called_once_with(container_port, subnets)
 
     @mock.patch(
         'kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif_nested_macvlan')
     def test_request_vif_port_create_failed(self, m_to_vif):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
         subnets = mock.sentinel.subnets
         security_groups = mock.sentinel.security_groups
 
-        port_request = mock.sentinel.port_request
+        port_request = {'foo': mock.sentinel.port_request}
         m_driver._get_port_request.return_value = port_request
-        neutron.create_port.side_effect = n_exc.NeutronClientException
+        os_net.create_port.side_effect = o_exc.SDKException
 
-        self.assertRaises(n_exc.NeutronClientException, cls.request_vif,
+        self.assertRaises(o_exc.SDKException, cls.request_vif,
                           m_driver, pod, project_id, subnets, security_groups)
         m_driver._get_port_request.assert_called_once_with(
             pod, project_id, subnets, security_groups)
-        neutron.create_port.assert_called_once_with(port_request)
+        os_net.create_port.assert_called_once_with(**port_request)
         m_driver._try_update_port.assert_not_called()
         m_to_vif.assert_not_called()
 
@@ -92,7 +94,7 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
     def test_request_vif_parent_not_found(self, m_to_vif):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -100,28 +102,28 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         security_groups = mock.sentinel.security_groups
         container_mac = mock.sentinel.mac_address
         container_ip = mock.sentinel.ip_address
-        container_port = self._get_fake_port(mac_address=container_mac,
-                                             ip_address=container_ip)
+        container_port = fake.get_port_obj(mac_address=container_mac,
+                                           ip_address=container_ip)
 
         port_request = mock.sentinel.port_request
         m_driver._get_port_request.return_value = port_request
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
-        neutron.create_port.return_value = container_port
-        m_driver._get_parent_port.side_effect = n_exc.NeutronClientException
+        os_net.create_port.return_value = container_port
+        m_driver._get_parent_port.side_effect = o_exc.SDKException
 
-        self.assertRaises(n_exc.NeutronClientException, cls.request_vif,
+        self.assertRaises(o_exc.SDKException, cls.request_vif,
                           m_driver, pod, project_id, subnets, security_groups)
         m_driver._get_port_request.assert_called_once_with(
             pod, project_id, subnets, security_groups)
-        neutron.create_port.assert_not_called()
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        os_net.create_port.assert_not_called()
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._try_update_port.assert_not_called()
         m_to_vif.assert_not_called()
 
     def test_release_vif(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
         pod = mock.sentinel.pod
@@ -130,42 +132,44 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
 
         container_mac = mock.sentinel.mac_address
         container_ip = mock.sentinel.ip_address
-        container_port = self._get_fake_port(port_id, container_ip,
-                                             container_mac)
-        neutron.show_port.return_value = container_port
+        container_port = fake.get_port_obj(
+            port_id=port_id, ip_address=container_ip,
+            mac_address=container_mac)
+        os_net.get_port.return_value = container_port
 
-        vm_port = self._get_fake_port()
+        vm_port = fake.get_port_obj()
         m_driver._get_parent_port.return_value = vm_port
         m_driver._try_update_port.return_value = 0
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
 
         cls.release_vif(m_driver, pod, vif)
 
-        neutron.show_port.assert_called_once_with(port_id)
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        os_net.get_port.assert_called_once_with(port_id)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._try_update_port.assert_called_once()
-        neutron.delete_port.assert_called_once_with(vif.id)
+        os_net.delete_port.assert_called_once_with(vif.id,
+                                                   ignore_missing=False)
 
     def test_release_vif_not_found(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         vif = mock.Mock()
         vif.id = lib_utils.get_hash()
 
-        neutron.show_port.side_effect = n_exc.PortNotFoundClient
+        os_net.get_port.side_effect = o_exc.NotFoundException
 
-        self.assertRaises(n_exc.PortNotFoundClient, cls.release_vif,
+        self.assertRaises(o_exc.NotFoundException, cls.release_vif,
                           m_driver, pod, vif)
         m_driver._remove_from_allowed_address_pairs.assert_not_called()
-        neutron.delete_port.assert_not_called()
+        os_net.delete_port.assert_not_called()
 
     def test_release_vif_parent_not_found(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
         pod = mock.sentinel.pod
@@ -174,26 +178,27 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
 
         container_mac = mock.sentinel.mac_address
         container_ip = mock.sentinel.ip_address
-        container_port = self._get_fake_port(port_id, container_ip,
-                                             container_mac)
-        neutron.show_port.return_value = container_port
+        container_port = fake.get_port_obj(
+            port_id=port_id, ip_address=container_ip,
+            mac_address=container_mac)
+        os_net.get_port.return_value = container_port
 
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
-        m_driver._get_parent_port.side_effect = n_exc.NeutronClientException
+        m_driver._get_parent_port.side_effect = o_exc.SDKException
 
-        self.assertRaises(n_exc.NeutronClientException, cls.release_vif,
+        self.assertRaises(o_exc.SDKException, cls.release_vif,
                           m_driver, pod, vif)
-        neutron.show_port.assert_called_with(port_id)
-        self.assertEqual(neutron.show_port.call_count, 1)
-        m_driver._get_parent_port.assert_called_with(neutron, pod)
+        os_net.get_port.assert_called_with(port_id)
+        self.assertEqual(os_net.get_port.call_count, 1)
+        m_driver._get_parent_port.assert_called_with(pod)
         self.assertEqual(m_driver._get_parent_port.call_count, 1)
         m_driver._remove_from_allowed_address_pairs.assert_not_called()
-        neutron.delete_port.assert_not_called()
+        os_net.delete_port.assert_not_called()
 
     def test_release_vif_delete_failed(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
         pod = mock.sentinel.pod
@@ -202,22 +207,24 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
 
         container_mac = mock.sentinel.mac_address
         container_ip = mock.sentinel.ip_addresses
-        container_port = self._get_fake_port(port_id, container_ip,
-                                             container_mac)
-        neutron.show_port.return_value = container_port
-        neutron.delete_port.side_effect = n_exc.PortNotFoundClient
+        container_port = fake.get_port_obj(
+            port_id=port_id, ip_address=container_ip,
+            mac_address=container_mac)
+        os_net.get_port.return_value = container_port
+        os_net.delete_port.side_effect = o_exc.NotFoundException
 
-        vm_port = self._get_fake_port()
+        vm_port = fake.get_port_obj()
         m_driver._get_parent_port.return_value = vm_port
         m_driver._try_update_port.return_value = 0
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
 
         cls.release_vif(m_driver, pod, vif)
 
-        neutron.show_port.assert_called_once_with(port_id)
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        os_net.get_port.assert_called_once_with(port_id)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._try_update_port.assert_called_once()
-        neutron.delete_port.assert_called_once_with(vif.id)
+        os_net.delete_port.assert_called_once_with(vif.id,
+                                                   ignore_missing=False)
 
     @ddt.data((False), (True))
     def test_activate_vif(self, active_value):
@@ -235,10 +242,10 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
     def test_add_to_allowed_address_pairs(self, m_mac):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00' if m_mac else vm_port['mac_address']
         address_pairs = [
@@ -255,31 +262,31 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
              'mac_address': m_mac if m_mac else vm_port['mac_address']}
         )
 
-        cls._add_to_allowed_address_pairs(m_driver, neutron, vm_port,
+        cls._add_to_allowed_address_pairs(m_driver, vm_port,
                                           frozenset([ip_addr]), m_mac)
 
         m_driver._update_port_address_pairs.assert_called_once_with(
-            neutron, port_id, address_pairs, revision_number=1)
+            port_id, address_pairs, revision_number=9)
 
     def test_add_to_allowed_address_pairs_no_ip_addresses(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         self.assertRaises(k_exc.IntegrityError,
                           cls._add_to_allowed_address_pairs, m_driver,
-                          neutron, vm_port, frozenset())
+                          vm_port, frozenset())
 
     def test_add_to_allowed_address_pairs_same_ip(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
         address_pairs = [
             {'ip_address': '10.0.0.30',
              'mac_address': 'fa:16:3e:1b:30:00'},
@@ -292,19 +299,19 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         ip_addr = '10.0.0.30'
         address_pairs.append({'ip_address': ip_addr, 'mac_address': mac_addr})
 
-        cls._add_to_allowed_address_pairs(m_driver, neutron, vm_port,
+        cls._add_to_allowed_address_pairs(m_driver, vm_port,
                                           frozenset([ip_addr]), mac_addr)
 
         m_driver._update_port_address_pairs.assert_called_once_with(
-            neutron, port_id, address_pairs, revision_number=1)
+            port_id, address_pairs, revision_number=9)
 
     def test_add_to_allowed_address_pairs_already_present(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
         address_pairs = [
             {'ip_address': '10.0.0.30',
              'mac_address': 'fa:16:3e:1b:30:00'},
@@ -317,17 +324,17 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         ip_addr = '10.0.0.30'
 
         self.assertRaises(k_exc.AllowedAddressAlreadyPresent,
-                          cls._add_to_allowed_address_pairs, m_driver, neutron,
+                          cls._add_to_allowed_address_pairs, m_driver,
                           vm_port, frozenset([ip_addr]), mac_addr)
 
     @ddt.data((None), ('fa:16:3e:71:cb:80'))
     def test_remove_from_allowed_address_pairs(self, m_mac):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00' if m_mac else vm_port['mac_address']
         address_pairs = [
@@ -345,31 +352,31 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         )
 
         cls._remove_from_allowed_address_pairs(
-            m_driver, neutron, vm_port, frozenset([ip_addr]), m_mac)
+            m_driver, vm_port, frozenset([ip_addr]), m_mac)
 
         m_driver._update_port_address_pairs.assert_called_once_with(
-            neutron, port_id, address_pairs, revision_number=1)
+            port_id, address_pairs, revision_number=9)
 
     def test_remove_from_allowed_address_pairs_no_ip_addresses(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         self.assertRaises(k_exc.IntegrityError,
                           cls._remove_from_allowed_address_pairs, m_driver,
-                          neutron, vm_port, frozenset())
+                          vm_port, frozenset())
 
     @ddt.data((None), ('fa:16:3e:71:cb:80'))
     def test_remove_from_allowed_address_pairs_missing(self, m_mac):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00' if m_mac else vm_port['mac_address']
         address_pairs = [
@@ -385,19 +392,19 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         ip_addr = ['10.0.0.29', '10.0.0.28']
 
         cls._remove_from_allowed_address_pairs(
-            m_driver, neutron, vm_port, frozenset(ip_addr), m_mac)
+            m_driver, vm_port, frozenset(ip_addr), m_mac)
 
         m_driver._update_port_address_pairs.assert_called_once_with(
-            neutron, port_id, address_pairs, revision_number=1)
+            port_id, address_pairs, revision_number=9)
 
     @ddt.data((None), ('fa:16:3e:71:cb:80'))
     def test_remove_from_allowed_address_pairs_no_update(self, m_mac):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00' if m_mac else vm_port['mac_address']
         address_pairs = [
@@ -411,43 +418,43 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         ip_addr = ['10.0.0.29']
 
         cls._remove_from_allowed_address_pairs(
-            m_driver, neutron, vm_port, frozenset(ip_addr), m_mac)
+            m_driver, vm_port, frozenset(ip_addr), m_mac)
 
         m_driver._update_port_address_pairs.assert_not_called()
 
     def test_update_port_address_pairs(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
         pairs = mock.sentinel.allowed_address_pairs
 
-        cls._update_port_address_pairs(m_driver, neutron, port_id, pairs,
-                                       revision_number=1)
+        cls._update_port_address_pairs(m_driver, port_id, pairs,
+                                       revision_number=9)
 
-        neutron.update_port.assert_called_with(
+        os_net.update_port.assert_called_with(
             port_id,
-            {'port': {'allowed_address_pairs': pairs}},
-            revision_number=1)
+            allowed_address_pairs=pairs,
+            if_match='revision_number=9')
 
     def test_update_port_address_pairs_failure(self):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
         pairs = mock.sentinel.allowed_address_pairs
-        neutron.update_port.side_effect = n_exc.NeutronClientException
+        os_net.update_port.side_effect = o_exc.SDKException
 
-        self.assertRaises(n_exc.NeutronClientException,
-                          cls._update_port_address_pairs, m_driver, neutron,
-                          port_id, pairs, revision_number=1)
+        self.assertRaises(o_exc.SDKException,
+                          cls._update_port_address_pairs, m_driver,
+                          port_id, pairs, revision_number=9)
 
-        neutron.update_port.assert_called_with(
+        os_net.update_port.assert_called_with(
             port_id,
-            {'port': {'allowed_address_pairs': pairs}},
-            revision_number=1)
+            allowed_address_pairs=pairs,
+            if_match='revision_number=9')
 
     @mock.patch('kuryr_kubernetes.controller.drivers.nested_macvlan_vif.'
                 'NestedMacvlanPodVIFDriver._add_to_allowed_address_pairs')
@@ -455,10 +462,10 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00'
         address_pairs = [
@@ -472,8 +479,7 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         ip_addr = ['10.0.0.29']
         attempts = cls._try_update_port(m_driver, 3,
                                         cls._add_to_allowed_address_pairs,
-                                        neutron, vm_port, frozenset(ip_addr),
-                                        mac_addr)
+                                        vm_port, frozenset(ip_addr), mac_addr)
         self.assertEqual(attempts, 0)
         aaapf_mock.assert_called_once()
 
@@ -483,10 +489,10 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
         cls = nested_macvlan_vif.NestedMacvlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
         m_driver.lock = mock.MagicMock(spec=threading.Lock())
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         port_id = lib_utils.get_hash()
-        vm_port = self._get_fake_port(port_id)['port']
+        vm_port = fake.get_port_obj(port_id)
 
         mac_addr = 'fa:16:3e:1b:30:00'
         address_pairs = [
@@ -499,45 +505,8 @@ class TestNestedMacvlanPodVIFDriver(test_base.TestCase):
 
         ip_addr = ['10.0.0.29']
 
-        aaapf_mock.side_effect = n_exc.NeutronClientException
-        self.assertRaises(n_exc.NeutronClientException,
+        aaapf_mock.side_effect = o_exc.SDKException
+        self.assertRaises(o_exc.SDKException,
                           cls._try_update_port, m_driver, 1,
                           cls._add_to_allowed_address_pairs,
-                          neutron, vm_port, frozenset(ip_addr), mac_addr)
-
-    # TODO(garyloug) consider exending and moving to a parent class
-    def _get_fake_port(self, port_id=None, ip_address=None, mac_address=None):
-        fake_port = {
-            'port': {
-                "mac_address": "fa:16:3e:20:57:c4",
-                "fixed_ips": [],
-                "id": "07b21ebf-b105-4720-9f2e-95670c4032e4",
-                "allowed_address_pairs": [],
-                "revision_number": 1
-            }
-        }
-
-        if port_id:
-            fake_port['port']['id'] = port_id
-
-        if ip_address:
-            fake_port['port']['fixed_ips'].append({
-                "subnet_id": lib_utils.get_hash(),
-                "ip_address": ip_address
-            })
-
-        if mac_address:
-            fake_port['port']['mac_address'] = mac_address
-
-        return fake_port
-
-    def _get_fake_ports(self, ip_address, mac_address):
-        fake_port = self._get_fake_port(ip_address=ip_address,
-                                        mac_address=mac_address)
-        fake_port = fake_port['port']
-        fake_ports = {
-            'ports': [
-                fake_port
-            ]
-        }
-        return fake_ports
+                          vm_port, frozenset(ip_addr), mac_addr)
